@@ -235,9 +235,10 @@ public class CreateCDF implements CDFConstants{
 
           longitude.selectCacheSize(700L);
           longitude.setCompression(GZIP_COMPRESSION, new long[] {9});
-          longitude.setPadValue(new Short((short) -99));
+//          longitude.setPadValue(new Short((short) -99));
           longitude.setBlockingFactor(130L);
           longitude.setInitialRecords(20L);
+System.out.println("pad value="+longitude.getPadValue()+" status="+cdf.getStatus()+" confirm="+longitude.confirmPadValue());
 
           // Only applicable to compressed z variables
           longitude.selectReservePercent(15L);   
@@ -536,6 +537,7 @@ public class CreateCDF implements CDFConstants{
           /*********************/
           /*  Add String data  */
           /*********************/
+	  name.setPadValue(new String("abc"));
           String [] ndata = new String[2];
           ndata[0] = "abcd";
           ndata[1] = "bcdefghij";
@@ -697,7 +699,7 @@ public class CreateCDF implements CDFConstants{
           /*  Add tt2000 data  */
           /*********************/
           
-          String dateTime = new String("2008-12-31T23:59:58.123456789");
+          String dateTime = new String("2015-06-30T23:59:58.123456789");
           long [] nanosecs = new long[5];
           long nansecSinceJ2000;
           try {
