@@ -1891,6 +1891,23 @@ VISIBLE_PREFIX void CDF_TT2000_to_UTC_string PROTOARGs((
 VISIBLE_PREFIX long long CDF_TT2000_from_UTC_string PROTOARGs((
   char *string
 ));
+VISIBLE_PREFIX long long computeTT2000withBasedLeapDay PROTOARGs((
+  long yy, long mm, long dd, long hh, long mn, long ss, long ms,
+  long us, long ns, int yymmdd
+));
+VISIBLE_PREFIX void breakdownTT2000withBasedLeapDay PROTOARGs((
+  long long tt2000, int yymmdd, 
+  long *yy, long *mm, long *dd, long *hh, long *mn, long *ss,
+  long *ms, long *us, long *ns
+));
+VISIBLE_PREFIX long long parseTT2000withBasedLeapDay PROTOARGs((
+  char *tt2000, int yymmdd
+));
+VISIBLE_PREFIX void encodeTT2000withBasedLeapDay PROTOARGs((
+  long long tt2000,
+  int yymmdd,
+  char *string
+));
 VISIBLE_PREFIX void CDFClearLeapSecondsTable PROTOARGs(());
 VISIBLE_PREFIX void CDFfillLeapSecondsTable PROTOARGs((
   double **table
