@@ -1,5 +1,5 @@
 /******************************************************************************
-* Copyright 1996-2013 United States Government as represented by the
+* Copyright 1996-2014 United States Government as represented by the
 * Administrator of the National Aeronautics and Space Administration.
 * All Rights Reserved.
 ******************************************************************************/
@@ -273,6 +273,7 @@
 #  include <smgdef.h>
 #  include <ssdef.h>
 #  include <stat.h>
+#  include <unistd.h>
 #  include <lib$routines.h>
 #  if defined(VARARGS)
 #    include <varargs.h>
@@ -469,7 +470,7 @@ typedef sChar sByte;
 #if defined(STAT)
 #  undef STAT
 #endif
-#define STAT stat
+/* #define STAT stat */
 #define FOPEN fopen
 #define FWRITE fwrite
 #define FREAD fread
@@ -487,6 +488,7 @@ typedef sChar sByte;
 #    define FREAD64 _read
 #    define FCLOSE64 _close
 #    define FLUSH64 _commit
+#    define STAT    _stat64
 #  else
 #    if defined(__MINGW32__) || defined(AIX)
        typedef off64_t OFF_T;
@@ -514,12 +516,18 @@ typedef sChar sByte;
 #      define FOPEN64 fopen64
 #    endif
 #    define FWRITE64 fwrite
-#    define FREAD64 fread
+#    define FREAD64  fread
 #    define FCLOSE64 fclose
-#    define FLUSH64 fflush
+#    define FLUSH64  fflush
+#    if defined(__CYGWIN__) || defined(vms)
+#      define STAT     stat
+#    else
+#      define STAT     stat64
+#    endif
 #  endif
 #else
    typedef long OFF_T;
+#  define STAT  stat
 #endif
 
 /******************************************************************************
@@ -779,6 +787,14 @@ ap = _apT_; \
 #define STRINGdataType(dataType) \
 (dataType == CDF_CHAR || dataType == CDF_UCHAR)
 
+/******************************************************************************
+* CDFepochDataType.
+*    Macro to check if one of the CDF epoch data types.
+******************************************************************************/
+
+#define CDFepochDataType(dataType) \
+(dataType == CDF_EPOCH || dataType == CDF_EPOCH16 || dataType == CDF_TIME_TT2000)
+
 /******************************************************************************
 * EPOCHdataType.
 *    Macro to check if an EPOCH data type.
@@ -932,10 +948,14 @@ typedef struct vSTATSstruct {
 #define VAR_ALLOCATEDTO(zOp) BOO(zOp,zVAR_ALLOCATEDTO_,rVAR_ALLOCATEDTO_)
 #define VAR_nINDEXLEVELS(zOp) BOO(zOp,zVAR_nINDEXLEVELS_,rVAR_nINDEXLEVELS_)
 #define VAR_RECORDS(zOp) BOO(zOp,zVAR_RECORDS_,rVAR_RECORDS_)
+#define VAR_RECORDS_RENUMBER(zOp) BOO(zOp,zVAR_RECORDS_RENUMBER_,rVAR_RECORDS_RENUMBER_)
 #define VAR_BLOCKINGFACTOR(zOp) BOO(zOp,zVAR_BLOCKINGFACTOR_, \
 					rVAR_BLOCKINGFACTOR_)
 #define VAR_RESERVEPERCENT(zOp) BOO(zOp,zVAR_RESERVEPERCENT_, \
 					rVAR_RESERVEPERCENT_)
+#define VAR_RECNUMBER(zOp) BOO(zOp,zVAR_RECNUMBER_,rVARs_RECNUMBER_)
+#define VAR_RECCOUNT(zOp) BOO(zOp,zVAR_RECCOUNT_,rVARs_RECCOUNT_)
+#define VAR_RECINTERVAL(zOp) BOO(zOp,zVAR_RECINTERVAL_,rVARs_RECINTERVAL_)
 
 #define VARs_MAXREC(zOp) BOO(zOp,zVARs_MAXREC_,rVARs_MAXREC_)
 #define VARs_RECDATA(zOp) BOO(zOp,zVARs_RECDATA_,rVARs_RECDATA_)
@@ -996,12 +1016,6 @@ VISIBLE_PREFIX Logical NegativeZeroReal8 PROTOARGs((double *value));
 VISIBLE_PREFIX Logical NegativeZeroReal4 PROTOARGs((float *value));
 VISIBLE_PREFIX void MakeUpperString PROTOARGs((char *string));
 VISIBLE_PREFIX void MakeLowerString PROTOARGs((char *string));
-VISIBLE_PREFIX void ROWtoCOL PROTOARGs((
-  void *iBuffer, void *oBuffer, long numDims, long dimSizes[], long nValueBytes
-));
-VISIBLE_PREFIX void COLtoROW PROTOARGs((
-  void *iBuffer, void *oBuffer, long numDims, long dimSizes[], long nValueBytes
-));
 VISIBLE_PREFIX void INCRindicesROW PROTOARGs((
   long numDims, long *dimSizes, long *indices
 ));
