@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF			CDF C interface for non-macro'ed functions
 *				of the Standard Interface.
@@ -34,6 +39,8 @@
 *                               CDFgetVarAllRecordsByVarName,
 *                               CDFgetVarRangeRecordsByVarName functions, and
 *                               a set of similar functions for put operations.
+*   V3.5  28-May-12, M. Liu     Added CDFinsertVarRecordsByVarID and
+*                               CDFinsertVarRecordsByVarName.
 *
 ******************************************************************************/
 
@@ -498,9 +505,9 @@ void    *buffer;        /* Out -- Buffer for holding data. */
   varNums[0] = varNum;
 
   if (!sX(CDFlib(SELECT_, CDF_, id,
-                          BOO((zVar == 1),zVARs_RECNUMBER_,rVARs_RECNUMBER_),
+                          BOO((zVar==1),zVARs_RECNUMBER_,rVARs_RECNUMBER_),
                           recNum,
-                 GET_, BOO((zVar == 1),zVARs_RECDATA_,rVARs_RECDATA_), numVars,
+                 GET_, BOO((zVar==1),zVARs_RECDATA_,rVARs_RECDATA_), numVars,
                        varNums, buffer,
                  NULL_), &pStatus)) return pStatus;
   return pStatus;
@@ -526,9 +533,9 @@ void    *buffer;        /* In -- Buffer for holding data. */
   varNums[0] = varNum;
 
   if (!sX(CDFlib(SELECT_, CDF_, id,
-                          BOO((zVar == 1),zVARs_RECNUMBER_,rVARs_RECNUMBER_),
+                          BOO((zVar==1),zVARs_RECNUMBER_,rVARs_RECNUMBER_),
                           recNum,
-                 PUT_, BOO((zVar == 1),zVARs_RECDATA_,rVARs_RECDATA_), numVars,
+                 PUT_, BOO((zVar==1),zVARs_RECDATA_,rVARs_RECDATA_), numVars,
                        varNums, buffer,
                  NULL_), &pStatus)) return pStatus;
   return pStatus;
@@ -568,19 +575,19 @@ void    *buffer;        /* Out -- Buffer for holding data. */
     intervals[i] = 1L;
   }
   if (!sX(CDFlib(SELECT_, CDF_, id,
-                          BOO((zVar == 1),zVAR_,rVAR_), varNum,
-                          BOO((zVar == 1),zVAR_RECNUMBER_,rVARs_RECNUMBER_),0L,
-                          BOO((zVar == 1),zVAR_RECCOUNT_,rVARs_RECCOUNT_),
+                          BOO((zVar==1),zVAR_,rVAR_), varNum,
+                          BOO((zVar==1),zVAR_RECNUMBER_,rVARs_RECNUMBER_),0L,
+                          BOO((zVar==1),zVAR_RECCOUNT_,rVARs_RECCOUNT_),
                           lastRec+1,
-                          BOO((zVar == 1),zVAR_RECINTERVAL_,rVARs_RECINTERVAL_),
+                          BOO((zVar==1),zVAR_RECINTERVAL_,rVARs_RECINTERVAL_),
                           1L,
-                          BOO((zVar == 1),zVAR_DIMINDICES_,rVARs_DIMINDICES_),
+                          BOO((zVar==1),zVAR_DIMINDICES_,rVARs_DIMINDICES_),
                           indices,
-                          BOO((zVar == 1),zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_),
+                          BOO((zVar==1),zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_),
                           dimSizes,
-                          BOO((zVar == 1),zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_),
+                          BOO((zVar==1),zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_),
                           intervals,
-                 GET_, BOO((zVar == 1),zVAR_HYPERDATA_,rVAR_HYPERDATA_),buffer,
+                 GET_, BOO((zVar==1),zVAR_HYPERDATA_,rVAR_HYPERDATA_),buffer,
                  NULL_), &pStatus)) return pStatus;
   return pStatus;
 }
@@ -598,44 +605,15 @@ void    *buffer;        /* Out -- Buffer for holding data. */
 
 {
   CDFstatus pStatus = CDF_OK;
-  int i, zVar;
-  long varNum, lastRec, numDims;
-  long indices[CDF_MAX_DIMS], intervals[CDF_MAX_DIMS], dimSizes[CDF_MAX_DIMS];
+  int zVar;
+  long varNum;
 
   varNum = CDFgetVarNum(id, varName);
   if (varNum < 0) return NO_SUCH_VAR;
   if (CDFconfirmVarExistence(id, 1, varName) == CDF_OK) zVar = 1;
   else zVar = 0;
-  if (zVar == 1) {
-    if (!sX(CDFgetzVarNumDims(id,varNum,&numDims), &pStatus)) return pStatus;
-    if (!sX(CDFgetzVarDimSizes(id,varNum,dimSizes), &pStatus)) return pStatus;
-    if (!sX(CDFgetVarMaxWrittenRecNum(id,1,varNum,&lastRec), &pStatus))
-      return pStatus;
-  } else {
-    if (!sX(CDFgetrVarsNumDims(id,&numDims), &pStatus)) return pStatus;
-    if (!sX(CDFgetrVarsDimSizes(id,dimSizes), &pStatus)) return pStatus;
-    if (!sX(CDFgetVarMaxWrittenRecNum(id,0,varNum,&lastRec), &pStatus))
-      return pStatus;
-  }
-  for (i = 0; i < (int) numDims; ++i) {
-    indices[i] = 0L;
-    intervals[i] = 1L;
-  }
-  if (!sX(CDFlib(SELECT_, CDF_, id,
-                          BOO((zVar == 1),zVAR_,rVAR_), varNum,
-                          BOO((zVar == 1),zVAR_RECNUMBER_,rVARs_RECNUMBER_),0L,
-                          BOO((zVar == 1),zVAR_RECCOUNT_,rVARs_RECCOUNT_),
-                          lastRec+1,
-                          BOO((zVar == 1),zVAR_RECINTERVAL_,rVARs_RECINTERVAL_),
-                          1L,
-                          BOO((zVar == 1),zVAR_DIMINDICES_,rVARs_DIMINDICES_),
-                          indices,
-                          BOO((zVar == 1),zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_),
-                          dimSizes,
-                          BOO((zVar == 1),zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_),
-                          intervals,
-                 GET_, BOO((zVar == 1),zVAR_HYPERDATA_,rVAR_HYPERDATA_),buffer,
-                 NULL_), &pStatus)) return pStatus;
+  if (!sX(CDFgetVarAllRecordsByVarID(id,zVar,varNum,buffer), &pStatus))
+    return pStatus;
   return pStatus;
 }
 
@@ -645,18 +623,18 @@ void    *buffer;        /* Out -- Buffer for holding data. */
 * staring from record 0.
 ******************************************************************************/
   
-VISIBLE_PREFIX CDFstatus CDFputVarAllRecordsByVarID (id, zVar, varNum, numRec,
+VISIBLE_PREFIX CDFstatus CDFputVarAllRecordsByVarID (id, zVar, varNum, numRecs,
                                                      buffer)
 CDFid   id;             /* In -- CDF id. */
 int     zVar;           /* In -- Flag for zVariable. */
 long    varNum;         /* In -- Variable number. */
-long    numRec;         /* In -- Number of Records to write. */
+long    numRecs;        /* In -- Number of Records to write. */
 void    *buffer;        /* In -- Buffer for holding data. */ 
   
 { 
   CDFstatus pStatus = CDF_OK;
   int i;
-  long numDims;
+  long numDims, maxRec;
   long indices[CDF_MAX_DIMS], intervals[CDF_MAX_DIMS], dimSizes[CDF_MAX_DIMS];
   
   if (zVar == 1) { 
@@ -670,21 +648,31 @@ void    *buffer;        /* In -- Buffer for holding data. */
     indices[i] = 0L;
     intervals[i] = 1L;
   }                       
+  if (!sX(CDFgetVarMaxWrittenRecNum(id,zVar,varNum,&maxRec),&pStatus))
+    return pStatus;
   if (!sX(CDFlib(SELECT_, CDF_, id, 
-                          BOO((zVar == 1),zVAR_,rVAR_), varNum,
-                          BOO((zVar == 1),zVAR_RECNUMBER_,rVARs_RECNUMBER_),0L,
-                          BOO((zVar == 1),zVAR_RECCOUNT_,rVARs_RECCOUNT_),
-                          numRec,
-                          BOO((zVar == 1),zVAR_RECINTERVAL_,rVARs_RECINTERVAL_),
+                          BOO((zVar==1),zVAR_,rVAR_), varNum,
+                          BOO((zVar==1),zVAR_RECNUMBER_,rVARs_RECNUMBER_),0L,
+                          BOO((zVar==1),zVAR_RECCOUNT_,rVARs_RECCOUNT_),
+                          numRecs,
+                          BOO((zVar==1),zVAR_RECINTERVAL_,rVARs_RECINTERVAL_),
                           1L,
-                          BOO((zVar == 1),zVAR_DIMINDICES_,rVARs_DIMINDICES_),
+                          BOO((zVar==1),zVAR_DIMINDICES_,rVARs_DIMINDICES_),
                           indices,
-                          BOO((zVar == 1),zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_),
+                          BOO((zVar==1),zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_),
                           dimSizes,
-                          BOO((zVar == 1),zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_),
+                          BOO((zVar==1),zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_),
                           intervals,
-                 PUT_, BOO((zVar == 1),zVAR_HYPERDATA_,rVAR_HYPERDATA_),buffer,
+                 PUT_, BOO((zVar==1),zVAR_HYPERDATA_,rVAR_HYPERDATA_),buffer,
                  NULL_), &pStatus)) return pStatus;
+  /****************************************************************************
+  * Update maximum record numbers.
+  ****************************************************************************/
+  if (maxRec > (numRecs-1)) {
+    if (!sX(CDFlib(DELETE_, BOO((zVar==1),zVAR_RECORDS_,rVAR_RECORDS_),
+                            numRecs, maxRec,
+                   NULL_), &pStatus)) return pStatus;
+  }
   return pStatus;
 }
 
@@ -694,49 +682,24 @@ void    *buffer;        /* In -- Buffer for holding data. */
 * from record 0.
 ******************************************************************************/
 
-VISIBLE_PREFIX CDFstatus CDFputVarAllRecordsByVarName (id, varName, numRec,
+VISIBLE_PREFIX CDFstatus CDFputVarAllRecordsByVarName (id, varName, numRecs,
                                                        buffer)
 CDFid   id;             /* In -- CDF id. */
 char    *varName;       /* In -- Variable name. */
-long    numRec;         /* In -- Number of Records to write. */
+long    numRecs;        /* In -- Number of Records to write. */
 void    *buffer;        /* In -- Buffer for holding data. */
 
 {
   CDFstatus pStatus = CDF_OK;
-  int i, zVar;
-  long varNum, numDims;
-  long indices[CDF_MAX_DIMS], intervals[CDF_MAX_DIMS], dimSizes[CDF_MAX_DIMS];
+  int zVar;
+  long varNum;
 
   varNum = CDFgetVarNum(id, varName);
   if (varNum < 0) return NO_SUCH_VAR;
   if (CDFconfirmVarExistence(id, 1, varName) == CDF_OK) zVar = 1;
   else zVar = 0;
-  if (zVar == 1) {
-    if (!sX(CDFgetzVarNumDims(id,varNum,&numDims), &pStatus)) return pStatus;
-    if (!sX(CDFgetzVarDimSizes(id,varNum,dimSizes), &pStatus)) return pStatus;
-  } else {
-    if (!sX(CDFgetrVarsNumDims(id,&numDims), &pStatus)) return pStatus;
-    if (!sX(CDFgetrVarsDimSizes(id,dimSizes), &pStatus)) return pStatus;
-  }
-  for (i = 0; i < (int) numDims; ++i) {
-    indices[i] = 0L;
-    intervals[i] = 1L;
-  }
-  if (!sX(CDFlib(SELECT_, CDF_, id,
-                          BOO((zVar == 1),zVAR_,rVAR_), varNum,
-                          BOO((zVar == 1),zVAR_RECNUMBER_,rVARs_RECNUMBER_),0L,
-                          BOO((zVar == 1),zVAR_RECCOUNT_,rVARs_RECCOUNT_),
-                          numRec,
-                          BOO((zVar == 1),zVAR_RECINTERVAL_,rVARs_RECINTERVAL_),
-                          1L,
-                          BOO((zVar == 1),zVAR_DIMINDICES_,rVARs_DIMINDICES_),
-                          indices,
-                          BOO((zVar == 1),zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_),
-                          dimSizes, 
-                          BOO((zVar == 1),zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_),
-                          intervals,
-                 PUT_, BOO((zVar == 1),zVAR_HYPERDATA_,rVAR_HYPERDATA_),buffer,
-                 NULL_), &pStatus)) return pStatus;
+  if (!sX(CDFputVarAllRecordsByVarID(id,zVar,varNum,numRecs,buffer), &pStatus))
+    return pStatus;
   return pStatus;
 }
 
@@ -774,20 +737,20 @@ void    *buffer;        /* Out -- Buffer for holding data. */
     intervals[i] = 1L;    
   }                       
   if (!sX(CDFlib(SELECT_, CDF_, id, 
-                          BOO((zVar == 1),zVAR_,rVAR_), varNum,
-                          BOO((zVar == 1),zVAR_RECNUMBER_,rVARs_RECNUMBER_),
+                          BOO((zVar==1),zVAR_,rVAR_), varNum,
+                          BOO((zVar==1),zVAR_RECNUMBER_,rVARs_RECNUMBER_),
                           startRec,
-                          BOO((zVar == 1),zVAR_RECCOUNT_,rVARs_RECCOUNT_),
+                          BOO((zVar==1),zVAR_RECCOUNT_,rVARs_RECCOUNT_),
                           stopRec-startRec+1,
-                          BOO((zVar == 1),zVAR_RECINTERVAL_,rVARs_RECINTERVAL_),
+                          BOO((zVar==1),zVAR_RECINTERVAL_,rVARs_RECINTERVAL_),
                           1L,
-                          BOO((zVar == 1),zVAR_DIMINDICES_,rVARs_DIMINDICES_),
+                          BOO((zVar==1),zVAR_DIMINDICES_,rVARs_DIMINDICES_),
                           indices,
-                          BOO((zVar == 1),zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_),
+                          BOO((zVar==1),zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_),
                           dimSizes,
-                          BOO((zVar == 1),zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_),
+                          BOO((zVar==1),zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_),
                           intervals,
-                 GET_, BOO((zVar == 1),zVAR_HYPERDATA_,rVAR_HYPERDATA_),buffer,
+                 GET_, BOO((zVar==1),zVAR_HYPERDATA_,rVAR_HYPERDATA_),buffer,
                  NULL_), &pStatus)) return pStatus;
   return pStatus;
 }
@@ -809,41 +772,15 @@ void    *buffer;        /* Out -- Buffer for holding data. */
 
 {
   CDFstatus pStatus = CDF_OK;
-  int i, zVar;
-  long varNum, numDims;
-  long indices[CDF_MAX_DIMS], intervals[CDF_MAX_DIMS], dimSizes[CDF_MAX_DIMS];
+  int  zVar;
+  long varNum;
 
   varNum = CDFgetVarNum(id, varName);
   if (varNum < 0) return NO_SUCH_VAR;
   if (CDFconfirmVarExistence(id, 1, varName) == CDF_OK) zVar = 1;
   else zVar = 0;
-  if (zVar == 1) {
-    if (!sX(CDFgetzVarNumDims(id,varNum,&numDims), &pStatus)) return pStatus;
-    if (!sX(CDFgetzVarDimSizes(id,varNum,dimSizes), &pStatus)) return pStatus;
-  } else {
-    if (!sX(CDFgetrVarsNumDims(id,&numDims), &pStatus)) return pStatus;
-    if (!sX(CDFgetrVarsDimSizes(id,dimSizes), &pStatus)) return pStatus;
-  }
-  for (i = 0; i < (int) numDims; ++i) {
-    indices[i] = 0L;
-    intervals[i] = 1L;
-  }
-  if (!sX(CDFlib(SELECT_, CDF_, id,
-                          BOO((zVar == 1),zVAR_,rVAR_), varNum,
-                          BOO((zVar == 1),zVAR_RECNUMBER_,rVARs_RECNUMBER_),
-                          startRec,
-                          BOO((zVar == 1),zVAR_RECCOUNT_,rVARs_RECCOUNT_),
-                          stopRec-startRec+1,
-                          BOO((zVar == 1),zVAR_RECINTERVAL_,rVARs_RECINTERVAL_),
-                          1L,
-                          BOO((zVar == 1),zVAR_DIMINDICES_,rVARs_DIMINDICES_),
-                          indices,
-                          BOO((zVar == 1),zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_),
-                          dimSizes,
-                          BOO((zVar == 1),zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_),
-                          intervals,
-                 GET_, BOO((zVar == 1),zVAR_HYPERDATA_,rVAR_HYPERDATA_),buffer,
-                 NULL_), &pStatus)) return pStatus;
+  if (!sX(CDFgetVarRangeRecordsByVarID(id,zVar,varNum,startRec,stopRec,
+                                       buffer), &pStatus)) return pStatus;
   return pStatus;
 }
 
@@ -880,20 +817,20 @@ void    *buffer;        /* In -- Buffer for holding data. */
     intervals[i] = 1L;    
   }                       
   if (!sX(CDFlib(SELECT_, CDF_, id,
-                          BOO((zVar == 1),zVAR_,rVAR_), varNum,
-                          BOO((zVar == 1),zVAR_RECNUMBER_,rVARs_RECNUMBER_),
+                          BOO((zVar==1),zVAR_,rVAR_), varNum,
+                          BOO((zVar==1),zVAR_RECNUMBER_,rVARs_RECNUMBER_),
                           startRec, 
-                          BOO((zVar == 1),zVAR_RECCOUNT_,rVARs_RECCOUNT_),
+                          BOO((zVar==1),zVAR_RECCOUNT_,rVARs_RECCOUNT_),
                           stopRec-startRec+1,
-                          BOO((zVar == 1),zVAR_RECINTERVAL_,rVARs_RECINTERVAL_),
+                          BOO((zVar==1),zVAR_RECINTERVAL_,rVARs_RECINTERVAL_),
                           1L,
-                          BOO((zVar == 1),zVAR_DIMINDICES_,rVARs_DIMINDICES_),
+                          BOO((zVar==1),zVAR_DIMINDICES_,rVARs_DIMINDICES_),
                           indices,
-                          BOO((zVar == 1),zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_),
+                          BOO((zVar==1),zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_),
                           dimSizes,
-                          BOO((zVar == 1),zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_),
+                          BOO((zVar==1),zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_),
                           intervals,
-                 PUT_, BOO((zVar == 1),zVAR_HYPERDATA_,rVAR_HYPERDATA_),buffer,
+                 PUT_, BOO((zVar==1),zVAR_HYPERDATA_,rVAR_HYPERDATA_),buffer,
                  NULL_), &pStatus)) return pStatus;
   return pStatus;
 }
@@ -914,44 +851,170 @@ void    *buffer;        /* In -- Buffer for holding data. */
 
 {
   CDFstatus pStatus = CDF_OK;
-  int i, zVar;
-  long varNum, numDims;
-  long indices[CDF_MAX_DIMS], intervals[CDF_MAX_DIMS], dimSizes[CDF_MAX_DIMS];
+  int zVar;
+  long varNum;
 
   varNum = CDFgetVarNum(id, varName);
   if (varNum < 0) return NO_SUCH_VAR;
   if (CDFconfirmVarExistence(id, 1, varName) == CDF_OK) zVar = 1;
   else zVar = 0;
+  if (!sX(CDFputVarRangeRecordsByVarID(id,zVar,varNum,startRec,stopRec,buffer),
+                                       &pStatus)) return pStatus;
+  return pStatus;
+}
+
+/******************************************************************************
+* CDFinsertVarRecordsByVarID.                          
+* Insert a number of records for a given variable (by zVar flag and id).
+******************************************************************************/
+
+VISIBLE_PREFIX CDFstatus CDFinsertVarRecordsByVarID (id, zVar, varNum,
+                                                     startRec, numRecs,
+                                                     buffer)
+CDFid   id;             /* In -- CDF id. */
+int     zVar;           /* In -- Flag for zVariable. */
+long    varNum;         /* In -- Variable number. */
+long    startRec;       /* In -- Starting record number to insert. */
+long    numRecs;        /* In -- The number of records to insert. */
+void    *buffer;        /* In -- Buffer for holding data. */
+  
+{ 
+  CDFstatus pStatus = CDF_OK;
+  int i;
+  long numDims, lastRec, dataType;
+  long indices[CDF_MAX_DIMS], intervals[CDF_MAX_DIMS], dimSizes[CDF_MAX_DIMS];
+  long sparseness;
+
+  if (numRecs < 1) return CDF_OK;
   if (zVar == 1) {
+    if (!sX(CDFgetVarSparseRecords(id,1,varNum,&sparseness), &pStatus))
+      return pStatus;
+    if (sparseness != NO_SPARSERECORDS) return CANNOT_INSERT_RECORDS;
     if (!sX(CDFgetzVarNumDims(id,varNum,&numDims), &pStatus)) return pStatus;
+    if (!sX(CDFgetVarDataType(id,1,varNum,&dataType), &pStatus)) return pStatus;
     if (!sX(CDFgetzVarDimSizes(id,varNum,dimSizes), &pStatus)) return pStatus;
+    if (!sX(CDFgetVarMaxWrittenRecNum(id,1,varNum,&lastRec), &pStatus))
+      return pStatus;
   } else {
+    if (!sX(CDFgetVarSparseRecords(id,0,varNum,&sparseness), &pStatus))
+      return pStatus;
+    if (sparseness != NO_SPARSERECORDS) return CANNOT_INSERT_RECORDS;
     if (!sX(CDFgetrVarsNumDims(id,&numDims), &pStatus)) return pStatus;
+    if (!sX(CDFgetVarDataType(id,0,varNum,&dataType), &pStatus)) return pStatus;
     if (!sX(CDFgetrVarsDimSizes(id,dimSizes), &pStatus)) return pStatus;
+    if (!sX(CDFgetVarMaxWrittenRecNum(id,0,varNum,&lastRec), &pStatus))
+      return pStatus;
   }
   for (i = 0; i < (int) numDims; ++i) {
     indices[i] = 0L;
     intervals[i] = 1L;
   }
+  if (startRec <= lastRec) {
+    void *tmp;
+    long nRecordValues, startRecX, startRecY, recs, recs2;
+    int dimN;
+    for (dimN = 0, nRecordValues = 1; dimN < numDims; dimN++) {
+      nRecordValues *= dimSizes[dimN];
+    }
+    recs2 = recs = lastRec - startRec + 1;
+    while (recs > 0) {
+      tmp = cdf_AllocateMemory ((size_t)recs * nRecordValues *
+                                CDFelemSize(dataType),
+                                NULL);
+      if (tmp != NULL) break;
+      recs = (int) (0.5 * recs + 0.5); 
+    }
+    if (tmp == NULL) return BAD_MALLOC;
+    for (startRecX = lastRec - recs + 1; recs2 > 0;
+         recs2 -= recs, startRecX -= recs) {
+      if (startRecX < startRec) {
+        recs = recs - (startRec - startRecX);
+        startRecX = startRec;
+      }
+      if (!sX(CDFlib(SELECT_, CDF_, id,
+                              BOO((zVar==1),zVAR_,rVAR_), varNum,
+                              BOO((zVar==1),zVAR_RECNUMBER_,rVARs_RECNUMBER_),
+                              startRecX, 
+                              BOO((zVar==1),zVAR_RECCOUNT_,rVARs_RECCOUNT_),
+                              recs,
+                              BOO((zVar==1),zVAR_RECINTERVAL_,rVARs_RECINTERVAL_),
+                              1L,
+                              BOO((zVar==1),zVAR_DIMINDICES_,rVARs_DIMINDICES_),
+                              indices,
+                              BOO((zVar==1),zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_),
+                              dimSizes,
+                              BOO((zVar==1),zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_),
+                              intervals,
+                     GET_, BOO((zVar==1),zVAR_HYPERDATA_,rVAR_HYPERDATA_),tmp,
+                     NULL_), &pStatus)) return pStatus;
+      startRecY = startRecX + numRecs;
+      if (!sX(CDFlib(SELECT_, CDF_, id,
+                              BOO((zVar==1),zVAR_,rVAR_), varNum,
+                              BOO((zVar==1),zVAR_RECNUMBER_,rVARs_RECNUMBER_),
+                              startRecY, 
+                              BOO((zVar==1),zVAR_RECCOUNT_,rVARs_RECCOUNT_),
+                              recs,
+                              BOO((zVar==1),zVAR_RECINTERVAL_,rVARs_RECINTERVAL_),
+                              1L,
+                              BOO((zVar==1),zVAR_DIMINDICES_,rVARs_DIMINDICES_),
+                              indices,
+                              BOO((zVar==1),zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_),
+                              dimSizes,
+                              BOO((zVar==1),zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_),
+                              intervals,
+                     PUT_, BOO((zVar==1),zVAR_HYPERDATA_,rVAR_HYPERDATA_),tmp,
+                     NULL_), &pStatus)) return pStatus;
+      
+    }
+    cdf_FreeMemory (tmp, NULL);
+  }
   if (!sX(CDFlib(SELECT_, CDF_, id,
-                          BOO((zVar == 1),zVAR_,rVAR_), varNum,
-                          BOO((zVar == 1),zVAR_RECNUMBER_,rVARs_RECNUMBER_),
-                          startRec,
-                          BOO((zVar == 1),zVAR_RECCOUNT_,rVARs_RECCOUNT_),
-                          stopRec-startRec+1,
-                          BOO((zVar == 1),zVAR_RECINTERVAL_,rVARs_RECINTERVAL_),
+                          BOO((zVar==1),zVAR_,rVAR_), varNum,
+                          BOO((zVar==1),zVAR_RECNUMBER_,rVARs_RECNUMBER_),
+                          startRec, 
+                          BOO((zVar==1),zVAR_RECCOUNT_,rVARs_RECCOUNT_),
+                          numRecs,
+                          BOO((zVar==1),zVAR_RECINTERVAL_,rVARs_RECINTERVAL_),
                           1L,
-                          BOO((zVar == 1),zVAR_DIMINDICES_,rVARs_DIMINDICES_),
+                          BOO((zVar==1),zVAR_DIMINDICES_,rVARs_DIMINDICES_),
                           indices,
-                          BOO((zVar == 1),zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_),
+                          BOO((zVar==1),zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_),
                           dimSizes,
-                          BOO((zVar == 1),zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_),
+                          BOO((zVar==1),zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_),
                           intervals,
-                 PUT_, BOO((zVar == 1),zVAR_HYPERDATA_,rVAR_HYPERDATA_),buffer,
+                 PUT_, BOO((zVar==1),zVAR_HYPERDATA_,rVAR_HYPERDATA_),buffer,
                  NULL_), &pStatus)) return pStatus;
   return pStatus;
 }
 
+/******************************************************************************
+* CDFinsertVarRecordsByVarName.
+* Insert a number of records for a given variable (by its unique name).
+******************************************************************************/
+
+VISIBLE_PREFIX CDFstatus CDFinsertVarRecordsByVarName (id, varName,
+                                                       startRec, numRecs,
+                                                       buffer)
+CDFid   id;             /* In -- CDF id. */
+char    *varName;       /* In -- Variable name. */
+long    startRec;       /* In -- Starting record number to insert. */
+long    numRecs;        /* In -- The number of records to insert. */
+void    *buffer;        /* In -- Buffer for holding data. */
+
+{
+  CDFstatus pStatus = CDF_OK;
+  int  zVar;
+  long varNum;
+
+  varNum = CDFgetVarNum(id, varName);
+  if (varNum < 0) return NO_SUCH_VAR;
+  if (CDFconfirmVarExistence(id, 1, varName) == CDF_OK) zVar = 1;
+  else zVar = 0;
+  if (!sX(CDFinsertVarRecordsByVarID(id,zVar,varNum,startRec,numRecs,buffer),
+                                     &pStatus)) return pStatus;
+  return pStatus;
+}
+
 /******************************************************************************
 * CDFgetVarsRecordDatabyNames. 
 * Acquire a full record data for a given record for a set of the selected 
@@ -985,24 +1048,24 @@ void    *buffptr[];     /* Out -- Array of buffer pointers for holding data. */
                  NULL_), &pStatus)) return pStatus;
 
   totalSize = 0;
-  phyRecSize = (long *) cdf_AllocateMemory (numVars*sizeof(long)*2, NULL);
+  phyRecSize = (long *) cdf_AllocateMemory ((size_t)numVars*sizeof(long)*2, NULL);
   if (phyRecSize == NULL) return BAD_MALLOC;
   varNums = phyRecSize+numVars;
 
   for (i = 0; i < numVars; i++) {
-    if (!sX(CDFlib(GET_, BOO((zVar == 1),zVAR_NUMBER_,rVAR_NUMBER_), 
+    if (!sX(CDFlib(GET_, BOO((zVar==1),zVAR_NUMBER_,rVAR_NUMBER_), 
                          varNames[i], &varNums[i], 
                    NULL_), &pStatus)) return pStatus;
-    if (!sX(CDFlib(SELECT_, BOO((zVar == 1),zVAR_,rVAR_), varNums[i],
-                   GET_, BOO((zVar == 1),zVAR_DATATYPE_,rVAR_DATATYPE_), 
+    if (!sX(CDFlib(SELECT_, BOO((zVar==1),zVAR_,rVAR_), varNums[i],
+                   GET_, BOO((zVar==1),zVAR_DATATYPE_,rVAR_DATATYPE_), 
                          &dataType,
-                         BOO((zVar == 1),zVAR_NUMELEMS_,rVAR_NUMELEMS_), 
+                         BOO((zVar==1),zVAR_NUMELEMS_,rVAR_NUMELEMS_), 
                          &numElems,
-                         BOO((zVar == 1),zVAR_NUMDIMS_,rVARs_NUMDIMS_), 
+                         BOO((zVar==1),zVAR_NUMDIMS_,rVARs_NUMDIMS_), 
                          &numDims,
-                         BOO((zVar == 1),zVAR_DIMSIZES_,rVARs_DIMSIZES_), 
+                         BOO((zVar==1),zVAR_DIMSIZES_,rVARs_DIMSIZES_), 
                          dimSizes,
-                         BOO((zVar == 1),zVAR_DIMVARYS_,rVAR_DIMVARYS_), 
+                         BOO((zVar==1),zVAR_DIMVARYS_,rVAR_DIMVARYS_), 
                          dimVarys,
                    NULL_), &pStatus)) return pStatus;
     if (!sX(CDFlib(GET_, DATATYPE_SIZE_, dataType, &dataTypeSize,
@@ -1021,12 +1084,12 @@ void    *buffptr[];     /* Out -- Array of buffer pointers for holding data. */
     totalSize += phyRecSize[i];
   }
 
-  buffer = (void *) cdf_AllocateMemory (totalSize, NULL);
+  buffer = (void *) cdf_AllocateMemory ((size_t)totalSize, NULL);
   if (buffer == NULL) return BAD_MALLOC;
 
-  if (!sX(CDFlib(SELECT_, BOO((zVar == 1),zVARs_RECNUMBER_,rVARs_RECNUMBER_), 
+  if (!sX(CDFlib(SELECT_, BOO((zVar==1),zVARs_RECNUMBER_,rVARs_RECNUMBER_), 
                          recNum,
-                 GET_, BOO((zVar == 1),zVARs_RECDATA_,rVARs_RECDATA_), numVars,
+                 GET_, BOO((zVar==1),zVARs_RECDATA_,rVARs_RECDATA_), numVars,
                        varNums, buffer,
                  NULL_), &pStatus)) return pStatus;
 
@@ -1071,25 +1134,25 @@ void    *buffptr[];     /* In -- Array of buffer pointers for holding data. */
   if (!sX(CDFlib(SELECT_, CDF_, id,
                  NULL_), &pStatus)) return pStatus;
 
-  phyRecSize = (long *) cdf_AllocateMemory (numVars*sizeof(long)*2, NULL);
+  phyRecSize = (long *) cdf_AllocateMemory ((size_t)numVars*sizeof(long)*2, NULL);
   if (phyRecSize == NULL) return BAD_MALLOC;
   varNums = phyRecSize+numVars;
 
   totalSize = 0;
   for (i = 0; i < numVars; i++) {
-    if (!sX(CDFlib(GET_, BOO((zVar == 1),zVAR_NUMBER_,rVAR_NUMBER_), 
+    if (!sX(CDFlib(GET_, BOO((zVar==1),zVAR_NUMBER_,rVAR_NUMBER_), 
                          varNames[i], &varNums[i],
                    NULL_), &pStatus)) return pStatus;
-    if (!sX(CDFlib(SELECT_, BOO((zVar == 1),zVAR_,rVAR_), varNums[i],
-                   GET_, BOO((zVar == 1),zVAR_DATATYPE_,rVAR_DATATYPE_), 
+    if (!sX(CDFlib(SELECT_, BOO((zVar==1),zVAR_,rVAR_), varNums[i],
+                   GET_, BOO((zVar==1),zVAR_DATATYPE_,rVAR_DATATYPE_), 
                          &dataType,
-                         BOO((zVar == 1),zVAR_NUMELEMS_,rVAR_NUMELEMS_), 
+                         BOO((zVar==1),zVAR_NUMELEMS_,rVAR_NUMELEMS_), 
                          &numElems,
-                         BOO((zVar == 1),zVAR_NUMDIMS_,rVARs_NUMDIMS_), 
+                         BOO((zVar==1),zVAR_NUMDIMS_,rVARs_NUMDIMS_), 
                          &numDims,
-                         BOO((zVar == 1),zVAR_DIMSIZES_,rVARs_DIMSIZES_), 
+                         BOO((zVar==1),zVAR_DIMSIZES_,rVARs_DIMSIZES_), 
                          dimSizes,
-                         BOO((zVar == 1),zVAR_DIMVARYS_,rVAR_DIMVARYS_), 
+                         BOO((zVar==1),zVAR_DIMVARYS_,rVAR_DIMVARYS_), 
                          dimVarys,
                    NULL_), &pStatus)) return pStatus;
     if (!sX(CDFlib(GET_, DATATYPE_SIZE_, dataType, &dataTypeSize,
@@ -1108,7 +1171,7 @@ void    *buffptr[];     /* In -- Array of buffer pointers for holding data. */
     totalSize += phyRecSize[i];
   }
 
-  buffer = (void *) cdf_AllocateMemory (totalSize, NULL);
+  buffer = (void *) cdf_AllocateMemory ((size_t)totalSize, NULL);
   if (buffer == NULL) return BAD_MALLOC;
 
   offset = 0;
@@ -1117,9 +1180,9 @@ void    *buffptr[];     /* In -- Array of buffer pointers for holding data. */
     offset += phyRecSize[i];
   }
 
-  if (!sX(CDFlib(SELECT_, BOO((zVar == 1),zVARs_RECNUMBER_,rVARs_RECNUMBER_), 
+  if (!sX(CDFlib(SELECT_, BOO((zVar==1),zVARs_RECNUMBER_,rVARs_RECNUMBER_), 
                          recNum,
-                 PUT_, BOO((zVar == 1),zVARs_RECDATA_,rVARs_RECDATA_), numVars,
+                 PUT_, BOO((zVar==1),zVARs_RECDATA_,rVARs_RECDATA_), numVars,
                        varNums, buffer,
                  NULL_), &pStatus)) return pStatus;
 
@@ -1151,7 +1214,7 @@ long    *maxEntry;      /* Out -- Maximum g/r/zEntry number used. */
                  NULL_), &pStatus)) return pStatus;
   if (GLOBALscope(*scope) && (zEntry == 1)) return ILLEGAL_FOR_SCOPE;
   if (!sX(CDFlib(GET_, ATTR_NAME_, attrName,
-                       BOO((zEntry == 1),ATTR_MAXzENTRY_,
+                       BOO((zEntry==1),ATTR_MAXzENTRY_,
                            (BOO(GLOBALscope(*scope),ATTR_MAXgENTRY_,
                                 ATTR_MAXrENTRY_))), maxEntry,
                  NULL_), &pStatus)) return pStatus;
@@ -1259,3 +1322,36 @@ VISIBLE_PREFIX int CDFgetFileBackwardFlag ()
   else return 1;
 }
 
+/******************************************************************************
+* CDFdeleteRecordsForAll. 
+* Delete a range of records for all variables in a CDF.
+******************************************************************************/
+
+VISIBLE_PREFIX CDFstatus CDFdeleteRecordsForAll (id, startRec, endRec)
+CDFid   id;             /* In -- CDF id. */
+long    startRec;       /* In -- Starting record to delete. */
+long    endRec;         /* In -- Ending record to delete. */ 
+
+{ 
+  CDFstatus pStatus = CDF_OK;
+  long numrVars, numzVars;
+  int i;
+
+  if (!sX(CDFlib(SELECT_, CDF_, id, 
+                 GET_, CDF_NUMrVARS_, &numrVars,
+                       CDF_NUMzVARS_, &numzVars,
+                 NULL_), &pStatus)) return pStatus;
+  for (i = 0; i < (int) numrVars; i++) {
+    if (!sX(CDFlib(SELECT_, rVAR_, (long) i,
+                   DELETE_, rVAR_RECORDS_RENUMBER_, startRec, endRec,
+                   NULL_), &pStatus)) return pStatus;
+  }
+  for (i = 0; i < (int) numzVars; i++) {
+    if (!sX(CDFlib(SELECT_, zVAR_, (long) i,
+                   DELETE_, zVAR_RECORDS_RENUMBER_, startRec, endRec,
+                   NULL_), &pStatus)) return pStatus;
+  }
+  return CDF_OK;
+
+}
+
