@@ -218,7 +218,7 @@ struct CDFstruct *CDF;
 * it, the data will be properly preserved.
 ******************************************************************************/
 
-STATICforIDL CDFstatus UpdateDotCDF (CDF)
+VISIBLE_PREFIX CDFstatus UpdateDotCDF (CDF)
 struct CDFstruct *CDF;
 {
   CDFstatus pStatus = CDF_OK; int varN; struct VarStruct *Var; Logical zVar;
