@@ -519,14 +519,14 @@ switch (Va->item) {
       }
     }
     if (CDFgetValidate() && (!PriorTo ("3.5.0", version, release, increment))) {
-      if (!sX(ValidateCDFTT2000 (*id, (int) CDF->NzVars, (int) CDF->NrVars),
-              &pStatus)) {
-        AbortAccess (CDF, noUPDATE, noDELETE);
-        cdf_FreeMemory (CDF, NULL);
-        Cur->cdf = NULL;
-        *id = (CDFid) NULL;
-        return pStatus;
-      }
+        if (!sX(ValidateCDFTT2000 (*id, (int) CDF->NzVars, (int) CDF->NrVars),
+                &pStatus)) {
+          AbortAccess (CDF, noUPDATE, noDELETE);
+          cdf_FreeMemory (CDF, NULL);
+          Cur->cdf = NULL;
+          *id = (CDFid) NULL;
+          return pStatus;
+        }
     }
     AddOpenCDFsCount();
     break;
@@ -555,12 +555,18 @@ static CDFstatus ValidateCDFTT2000 (CDFid id, int zVars, int rVars)
   long j, count;
   long long tt2000 = DEFAULT_TT2000_PADVALUE;
   void *buffer;
-  long tt2000LastUpdated, numDims, dimSizes[CDF_MAX_DIMS];
+  long year, month, day, tt2000LastUpdated, numDims, dimSizes[CDF_MAX_DIMS];
   double yy, mm, dd, hh, mn, ss, ml, ms, ns;
 
   status = CDFgetLeapSecondLastUpdated (id, &tt2000LastUpdated);
   if (status != CDF_OK) return status;
-  if (tt2000LastUpdated == 0) return CDF_OK;
+#if defined(vms)
+  CDFgetLastDateinLeapSecondsTBL (&year, &month, &day);
+#else
+  CDFgetLastDateinLeapSecondsTable (&year, &month, &day);
+#endif
+  if (tt2000LastUpdated == 0 ||
+      tt2000LastUpdated == (10000*year+100*month+day)) return CDF_OK;
   for (i = 0; i < zVars; ++i) {
     status = CDFlib(SELECT_, zVAR_, (long) i,
                     GET_, zVAR_DATATYPE_, &dataType,
