@@ -1,5 +1,5 @@
 /******************************************************************************
-* Copyright 1996-2013 United States Government as represented by the
+* Copyright 1996-2014 United States Government as represented by the
 * Administrator of the National Aeronautics and Space Administration.
 * All Rights Reserved.
 ******************************************************************************/
@@ -236,7 +236,7 @@ struct STRINGstruct **ssh;		/* Pointer to head of STRINGstruct
 * returned.
 ******************************************************************************/
 
-STATICforIDL Logical Trailer (string, trailer)
+VISIBLE_PREFIX Logical Trailer (string, trailer)
 char *string;
 char *trailer;
 {
@@ -260,7 +260,7 @@ char *trailer;
 * including the terminating NUL character.
 ******************************************************************************/
 
-STATICforIDL char *strcpyX (dst, src, max)
+VISIBLE_PREFIX char *strcpyX (dst, src, max)
 char *dst;
 char *src;
 size_t max;
@@ -298,7 +298,7 @@ size_t max;
 * including the terminating NUL character.
 ******************************************************************************/
 
-STATICforIDL char *strcatX (dst, src, max)
+VISIBLE_PREFIX char *strcatX (dst, src, max)
 char *dst;
 char *src;
 size_t max;
@@ -334,7 +334,7 @@ size_t max;
 * concatenated. 
 ******************************************************************************/
 
-STATICforIDL char *catchrX (dst, src, max)
+VISIBLE_PREFIX char *catchrX (dst, src, max)
 char *dst;
 int src;
 size_t max;
@@ -358,7 +358,7 @@ size_t max;
 * destination string exceed a maximum length.
 ******************************************************************************/
 
-STATICforIDL char *prependX (dst, src, max)
+VISIBLE_PREFIX char *prependX (dst, src, max)
 char *dst;
 char *src;
 size_t max;
@@ -468,7 +468,7 @@ int length;
 * Printable.
 ******************************************************************************/
 
-STATICforIDL int Printable (c)
+VISIBLE_PREFIX int Printable (c)
 int c;
 {
   return BOO(IsASCII(c),isprint(c),FALSE);
@@ -478,7 +478,7 @@ int c;
 * UpperCase.
 ******************************************************************************/
 
-STATICforIDL int UpperCase (c)
+VISIBLE_PREFIX int UpperCase (c)
 int c;
 {
   return BOO(IsASCII(c),isupper(c),FALSE);
@@ -488,7 +488,7 @@ int c;
 * LowerCase.
 ******************************************************************************/
 
-STATICforIDL int LowerCase (c)
+VISIBLE_PREFIX int LowerCase (c)
 int c;
 {
   return BOO(IsASCII(c),islower(c),FALSE);
@@ -498,7 +498,7 @@ int c;
 * Alphabetic.
 ******************************************************************************/
 
-STATICforIDL int Alphabetic (c)
+VISIBLE_PREFIX int Alphabetic (c)
 int c;
 {
   return BOO(IsASCII(c),isalpha(c),FALSE);
@@ -508,7 +508,7 @@ int c;
 * Decimal.
 ******************************************************************************/
 
-STATICforIDL int Decimal (c)
+VISIBLE_PREFIX int Decimal (c)
 int c;
 {
   return BOO(IsASCII(c),isdigit(c),FALSE);
@@ -518,7 +518,7 @@ int c;
 * Spacing.
 ******************************************************************************/
 
-STATICforIDL int Spacing (c)
+VISIBLE_PREFIX int Spacing (c)
 int c;
 {
   return BOO(IsASCII(c),isspace(c),FALSE);
@@ -528,7 +528,7 @@ int c;
 * Visible.
 ******************************************************************************/
 
-STATICforIDL int Visible (c)
+VISIBLE_PREFIX int Visible (c)
 int c;
 {
   return BOO(IsASCII(c),isgraph(c),FALSE);
@@ -538,7 +538,7 @@ int c;
 * MakeLower.
 ******************************************************************************/
 
-STATICforIDL int MakeLower (c)
+VISIBLE_PREFIX int MakeLower (c)
 int c;
 {
   return BOO(UpperCase(c),(c | 0x20),c);
@@ -548,7 +548,7 @@ int c;
 * MakeUpper.
 ******************************************************************************/
 
-STATICforIDL int MakeUpper (c)
+VISIBLE_PREFIX int MakeUpper (c)
 int c;
 {
   return BOO(LowerCase(c),(c & 0x5F),c);
