@@ -101,6 +101,11 @@ static double **LTD = NULL;
 static long long *NST = NULL;
 static int entryCnt;
 
+static long doys1[] = {31,59,90,120,151,181,212,243,273,304,334,365};
+static long doys2[] = {31,60,91,121,152,182,213,244,274,305,335,366};
+static long daym1[] = {31,28,31,30,31,30,31,31,30,31,30,31};
+static long daym2[] = {31,29,31,30,31,30,31,31,30,31,30,31};
+
 static double JulianDay12h (long, long, long);
 static void DatefromJulianDay (double, long *, long *, long *);
 static int ValidateYMD (long, long , long);
@@ -113,6 +118,10 @@ static void LoadLeapSecondsTable ();
 static void LoadLeapNanoSecondsTable ();
 static void EPOCHbreakdownTT2000 (double, long *, long *, long *, long *,
                                   long *, long *);
+static int LeapSecondLastUpdatedinTable (int);
+static void FromUTCISO8601string (char *string, long *ly, long *lm, long *ld, 
+                                  long *lh, long *ln, long *ls,
+                                  long *ll, long *lu, long *la);
 
 static char *MonthToken (long month)
 {
@@ -635,6 +644,183 @@ va_dcl
   }
 }
 
+/******************************************************************************
+* computeTT2000withBasedLeapDay.
+* This function is similar to CDF_TT2000_from_UTC_parts (aka computeTT2000),
+* with an adjustment if necessary. It returns the TT2000 from UTC date/time
+* based on the passed last leap second day, while CDF_TT2000_from_UTC_parts
+* function is based on the last leap second updated day in the current leap
+* second table (LST). 
+* If the based leap second day is not a zero (0) or if the UTC date/time is 
+* extended over the next leap second day in the LST, one second(s) will be
+* adjusted (deduction), due to the newly added leap second(s) in the LST.
+* Otherwise, it returns the same value as CDF_TT2000_from_UTC_parts.
+******************************************************************************/
+
+VISIBLE_PREFIX long long computeTT2000withBasedLeapDay (long yy, long mm, 
+                                                        long dd, long hh,
+                                                        long mn, long ss,
+                                                        long ms, long us,
+                                                        long ns, int yymmdd)
+{
+  long long nanoSecSinceJ2000;
+  if (yy < 0 || mm < 0 || dd < 0 || hh < 0 || mn < 0 ||
+      ss < 0 || ms < 0 || us < 0 || ns < 0)
+    return ILLEGAL_TT2000_VALUE;
+  nanoSecSinceJ2000 = computeTT2000 ((double)yy, (double)mm, (double)dd,
+                                     (double)hh, (double)mn, (double)ss,
+                                     (double)ms, (double)us, (double)ns);
+  if (yymmdd <= 0 || (yy*10000+mm*100+dd) < (long)yymmdd ||
+      ((yy*10000+mm*100+dd) == (long)yymmdd && (hh*10000+mn*100+ss) < 235960))
+    return nanoSecSinceJ2000;
+  else {
+    int ix, iy;
+    ix = LeapSecondLastUpdatedinTable (yymmdd);
+    iy = LeapSecondLastUpdatedinTable ((int)yy*10000+(int)mm*100+(int)dd);
+    return (nanoSecSinceJ2000 - (long long)(iy-ix)*1000000000LL);
+  }
+}
+
+/******************************************************************************
+* breakdownTT2000withBasedLeapDay.
+* This function is similar to CDF_TT2000_to_UTC_parts (aka breakdownTT2000),
+* with an adjustment if necessary. It returns UTC date/time from TT2000 value,
+* based on the passed last leap second day, while CDF_TT2000_to_UTC_parts
+* function is based on the last leap second updated day in the current leap
+* second table (LST). 
+* If the based leap second day is zero (0), this routine functions just like
+* CDF_TT2000_to_UTC_parts. If the TT2000 value was computed based on the 
+* LST that was not up-to-date, one second(s) adjustment (addition) might be 
+* necessay if its value crosses over the next new leap second day in the LST. 
+******************************************************************************/
+
+VISIBLE_PREFIX void breakdownTT2000withBasedLeapDay (long long tt2000,
+                                                     int yymmdd, long *yy,
+                                                     long *mm, long *dd,
+                                                     long *hh, long *mn,
+                                                     long *ss, long *ms,
+                                                     long *us, long *ns)
+{
+  long long *tmpTT2000s;
+  double yyy, mmm, ddd, hhh, mmn, sss, mms, mus, mns;
+  int off, ix, iy, found;
+
+  if (yymmdd <= 0)
+    breakdownTT2000 (tt2000, &yyy, &mmm, &ddd, &hhh, &mmn, &sss, &mms, &mus,
+		     &mns);
+  else {
+    ix = LeapSecondLastUpdatedinTable (yymmdd);
+    off = entryCnt - ix - 1;
+    if (off == 0)
+      breakdownTT2000 (tt2000, &yyy, &mmm, &ddd, &hhh, &mmn, &sss, &mms,
+		       &mus, &mns);
+    else {
+      found = 0;
+      tmpTT2000s = (long long *) malloc (sizeof (long long) * off);
+      for (iy = 0; iy < off; ++iy)
+        tmpTT2000s[iy] = computeTT2000withBasedLeapDay (LTD[ix+1+iy][0],
+                                                        LTD[ix+1+iy][1],
+                                                        LTD[ix+1+iy][2],
+                                                        0.0, 0.0, 0.0, 0.0,
+                                                        0.0, 0.0, yymmdd);
+      for (iy = off; iy > 0; --iy) {
+	if (tt2000 >= tmpTT2000s[iy-1]) {
+	  breakdownTT2000 (tt2000+(long long)iy*1000000000LL, &yyy, &mmm,
+                           &ddd, &hhh, &mmn, &sss, &mms, &mus, &mns);
+          found = 1;
+	  break;
+	}
+      }
+      if (found == 0)
+        breakdownTT2000 (tt2000, &yyy, &mmm, &ddd, &hhh, &mmn, &sss, &mms,
+                         &mus, &mns);
+      free (tmpTT2000s);
+    }
+  }
+  *yy = (long) yyy;
+  *mm = (long) mmm;
+  *dd = (long) ddd;
+  *hh = (long) hhh;
+  *mn = (long) mmn;
+  *ss = (long) sss;
+  *ms = (long) mms;
+  *us = (long) mus;
+  *ns = (long) mns;
+}
+
+/******************************************************************************
+* encodeTT2000withBasedLeapDay. 
+* This function is similar to CDF_TT2000_to_UTC_string (aka encodeTT2000),
+* with an adjustment if necessary. It encodes TT2000 to UTC string in ISO 8601
+* form based on the passed last leap second day, while CDF_TT2000_to_UTC_string
+* function is based on the last leap second updated day in the current leap
+* second table (LST). 
+* If the based leap second day is zero (0), this routine functions just like
+* CDF_TT2000_to_UTC_string. If the TT2000 value was computed based on the 
+* LST that was not up-to-date, one second(s) adjustment (addition) might be 
+* necessay if its value crosses over the next new leap second day in the LST.
+******************************************************************************/
+
+VISIBLE_PREFIX void encodeTT2000withBasedLeapDay (long long tt2000,
+                                                  int yymmdd, char *string)
+{
+  long long *tmpTT2000s;
+  int off, ix, iy, found;
+  if (yymmdd <= 0) {
+    CDF_TT2000_to_UTC_string (tt2000, string);
+    return;
+  }
+  ix = LeapSecondLastUpdatedinTable (yymmdd);
+  off = entryCnt - ix - 1;
+  if (off == 0) {
+    CDF_TT2000_to_UTC_string (tt2000, string);
+    return;
+  } else {
+    found = 0;
+    tmpTT2000s = (long long *) malloc (sizeof (long long) * off);
+    for (iy = 0; iy < off; ++iy)
+      tmpTT2000s[iy] = computeTT2000withBasedLeapDay (LTD[ix+1+iy][0],
+                                                      LTD[ix+1+iy][1],
+                                                      LTD[ix+1+iy][2],
+                                                      0.0, 0.0, 0.0, 0.0,
+                                                      0.0, 0.0, yymmdd);
+    for (iy = off; iy > 0; --iy) {
+      if (tt2000 >= tmpTT2000s[iy-1]) {
+        CDF_TT2000_to_UTC_string (tt2000+(long long)iy*1000000000LL, string);
+        found = 1;
+        break;
+      }
+    }
+    if (found == 0)
+      CDF_TT2000_to_UTC_string (tt2000, string);
+    free (tmpTT2000s);
+  }
+  return;
+}
+
+/******************************************************************************
+* parseTT2000withBasedLeapDay.
+* This function is similar to CDF_TT2000_from_UTC_string (aka parseTT2000),
+* with an adjustment if necessary. It returns a TT2000 value from the UTC
+* string in ISO 8601 form based on the passed last leap second day, while
+* CDF_TT2000_from_UTC_string function is based on the last leap second updated
+* day in the current leap second table (LST). 
+* If the based leap second day is zero (0), this routine functions just like
+* CDF_TT2000_from_UTC_string. If the UTC string was encoded based on the 
+* LST that was not up-to-date, one second(s) adjustment (addition) might be 
+* necessay if its day crosses over the next new leap second day in the LST.
+******************************************************************************/
+
+VISIBLE_PREFIX long long parseTT2000withBasedLeapDay (char *tt2000, int yymmdd)
+{
+  long yy, mm, dd, hh, mn, ss, ms, us, ns;
+  FromUTCISO8601string (tt2000, &yy, &mm, &dd, &hh, &mn, &ss, &ms, &us, &ns);
+  return computeTT2000withBasedLeapDay ((double)yy, (double)mm, (double)dd,
+				        (double)hh, (double)mn, (double)ss,
+				        (double)ms, (double)us, (double)ns,
+				        yymmdd);
+}
+
 /******************************************************************************
 * CDF_TT2000_from_UTC_parts.
 ******************************************************************************/
@@ -658,6 +844,8 @@ va_dcl
   long lyl, lml, ldl, lhl, lnl, lsl, lll, lul, lal;
   double tmpy;
   long xx, xy, xm, xd;
+  int lyear;
+
 #if defined(STDARG)
   va_start (Va, dd);
 #else
@@ -897,6 +1085,28 @@ va_dcl
     return DEFAULT_TT2000_PADVALUE;
   if (!YearWithin(lyl) && !ValidateYMD(lyl,lml,ldl))
     return ILLEGAL_TT2000_VALUE;
+  lyear = (lyl & 3) == 0 && ((lyl % 25) != 0 || (lyl & 15) == 0);
+  if ((!lyear && ldl > 365) || (lyear && ldl > 366))
+    return ILLEGAL_TT2000_VALUE;
+  if ((!lyear && lml > 1 && ldl > daym1[(int)lml-1]) ||
+      (lyear && lml > 1 && ldl > daym2[(int)lml-1]))
+    return ILLEGAL_TT2000_VALUE;
+  if (lml <= 1 && ldl > 31) {
+     /* DOY is passed in. */
+     int ix;
+     if (lml == 0) lml = 1;
+     for (ix = 0; ix < 12; ++ix) {
+        if ((!lyear && ldl <= doys1[ix]) || (lyear && ldl <= doys2[ix])) {
+          if (ix == 0) break;
+          else {
+            lml = ix + 1;
+            if (!lyear) ldl = ldl - doys1[ix-1];
+            else ldl = ldl - doys2[ix-1];
+            break;
+          }
+        }
+     }
+  }
   iy = 10000000 * lml + 10000 * ldl + lyl;
   if (iy != currentDay) {
     currentDay = iy;
@@ -1222,7 +1432,7 @@ VISIBLE_PREFIX void ReduceOpenCDFsCount ()
 
 /******************************************************************************
 * ValidateTT2000.
-* Retuens 1 if the TT2000 data is determined to be created properly based on
+* Returns 1 if the TT2000 data is determined to be created properly based on
 * the info from the currently used leap second table, e.g., prior-to 2015/07/01
 * or leap second last updated being equal to the last entry in the table, or
 * the TT2000 time is within the next new leap second date.
@@ -1235,23 +1445,23 @@ VISIBLE_PREFIX void ReduceOpenCDFsCount ()
 * using the older table, which can be bad after 2015-07-01).
 ******************************************************************************/
 
-VISIBLE_PREFIX int ValidateTT2000 (int tt2000YYYYMMDD,
+VISIBLE_PREFIX int ValidateTT2000 (int yyyymmdd,
                                    int leapSecondLastUpdated)
 {
   int i, last, lastDate;
-  if (tt2000YYYYMMDD < 20150701) return 1;
+  if (yyyymmdd < 20150701) return 1;
   if (LTD == NULL) LoadLeapSecondsTable ();
   last = entryCnt - 1;
   lastDate = (int) (10000*LTD[last][0] + 100*LTD[last][1] + LTD[last][2]);
   if (leapSecondLastUpdated == lastDate) return 1;
   if (leapSecondLastUpdated > 0) {
-    if (tt2000YYYYMMDD < leapSecondLastUpdated) return 1;
+    if (yyyymmdd < leapSecondLastUpdated) return 1;
     if (lastDate > leapSecondLastUpdated) {
       for (i = last-1; i >= 0; --i) {
         if (leapSecondLastUpdated == (int) (10000*LTD[i][0] + 100*LTD[i][1] +
                                             LTD[i][2])) {
-          if (tt2000YYYYMMDD < (int) (10000*LTD[i+1][0] + 100*LTD[i+1][1] +
-                                      LTD[i+1][2])) return 1;
+          if (yyyymmdd < (int) (10000*LTD[i+1][0] + 100*LTD[i+1][1] +
+                                LTD[i+1][2])) return 1;
           else return -1;
         }
       }
@@ -1264,7 +1474,7 @@ VISIBLE_PREFIX int ValidateTT2000 (int tt2000YYYYMMDD,
 
 /******************************************************************************
 * ValidateLeapSecondLastUpdated.
-* Retuens 1 if the passed date, in YYYYMMDD, is in the leap second table.
+* Returns 1 if the passed date, in YYYYMMDD, is in the leap second table.
 * Otherwise, 0.
 ******************************************************************************/
 
@@ -1283,3 +1493,47 @@ VISIBLE_PREFIX int ValidateLeapSecondLastUpdated (int leapSecondLastUpdated)
   return 0;
 }
 
+/******************************************************************************
+* LeapSecondLastUpdatedinTable.
+* Returns the index in the leap second table (LST) for the passed leap second 
+* last updated day (in YYYYMMDD). If the day is not in the LST, the nearest
+* index from the table is returned, which has the day less than the passed
+* day. 
+******************************************************************************/
+    
+static int LeapSecondLastUpdatedinTable (int leapSecondLastUpdated)
+{       
+  int i, last;
+  if (LTD == NULL) LoadLeapSecondsTable ();
+  last = entryCnt - 1;
+  for (i = last; i >= 0; --i) {
+    if (leapSecondLastUpdated >= (int) (10000*LTD[i][0] + 100*LTD[i][1] +
+                                        LTD[i][2])) return i;
+  }     
+  return 0;
+}       
+
+static void FromUTCISO8601string (char *string, long *ly, long *lm, long *ld,
+                                  long *lh, long *ln, long *ls,
+                                  long *ll, long *lu, long *la)
+{
+  int format, len;
+  char *dot;
+
+  format = ScanUTCstring(string);
+  if (format == 3) { /* %4ld-%2ld-%2ldT%2ld:%2ld:%2ld.%9ld */
+    long tmp;
+    *ly = *lm = *ld = *lh = *ln = *ls = *ll = *lu = *la = 0L;
+    if (sscanf(string,"%4ld-%2ld-%2ldT%2ld:%2ld:%2ld.%9ld",
+        ly, lm, ld, lh, ln, ls, &tmp) != 7) return;
+    dot = strrchr (string, '.');
+    len = (int) strlen (dot+1);
+    if (len < 9) tmp = tmp * (long) pow (10.0, 9-len);
+    *ll = (long) (tmp / 1000000L);
+    tmp = tmp - *ll * 1000000L;
+    *lu = (long) (tmp / 1000L);
+    *la = (long) (tmp - *lu * 1000L);
+  }
+}
+
+
