@@ -489,40 +489,85 @@ void *buffer;
 size_t nBytes;
 vFILE *vFp;
 {
-  int tryN;
+  int tryN, pid;
+  FILE *fp;
 #if defined(__MWERKS__)
   int ii;
+#endif
+#if defined(win32)
+  char *temp;
 #endif
   /****************************************************************************
   * Create the scratch file if necessary.  If so, the current file path is
   * actually the scratch directory to be used.
   ****************************************************************************/
   if (vFp->fp == NULL) {
-    long i; char *tmpPath; size_t pathLength;
+    long i; char *tmpPath, *originalPath; size_t pathLength;
     pathLength = strlen(vFp->path) + 1 + 8 + 1 + EXT_LEN;
+    if (pathLength < DU_MAX_DIR_LEN) pathLength = DU_MAX_DIR_LEN;
     tmpPath = (char *) cdf_AllocateMemory ((size_t)pathLength + 1, NULL);
     if (tmpPath == NULL) return FALSE;
-    for (i = 1; i <= MAX_TMP; i++) {
-       strcpyX (tmpPath, vFp->path, 0);
-       AppendToDir (tmpPath, "");
-       snprintf (EofS(tmpPath), (size_t) pathLength + 1 - strlen(tmpPath),
-                 "TMP%05ld.%s", i, vFp->scratchExt);
-       if (!IsReg(tmpPath)) {
-	 FILE *fp = OpenFile (tmpPath, WRITE_PLUS_a_mode);
-	 if (fp == NULL) {
-	   cdf_FreeMemory (tmpPath, NULL);
-	   return FALSE;
-	 }
-	 cdf_FreeMemory (vFp->path, NULL);
-	 vFp->path = tmpPath;
-	 vFp->fp = fp;
-	 break;
-       }
+    originalPath = (char *) cdf_AllocateMemory ((size_t)pathLength + 1, NULL);
+    if (originalPath == NULL) return FALSE;
+    pid = GetMyPID();
+    if (pid == -1) {
+#if defined(win32)
+      pid = (int) _getpid();
+#else
+      pid = (int) getpid();
+#endif
+      SetMyPID (pid);
+      srand((unsigned int)pid*time(NULL));
     }
-    if (vFp->fp == NULL) {	   /* Hardly seems likely but we'll check... */
+    if (vFp->path != NULL && strlen(vFp->path) != 0) {
+      strcpyX (tmpPath, vFp->path, 0);
+      AppendToDir (tmpPath, "");
+    } else {
+      /* Use the known tmp direcotry */
+#if defined(linux) || defined(unix)
+      strcpyX (tmpPath, "/tmp/", 0);
+#else
+#  if defined(win32)
+      temp = getenv("TEMP");
+      if (temp != NULL) {
+        strcpyX (tmpPath, temp, 0);
+        strcatX (tmpPath, "\\", 0);
+      } else {
+        temp = getenv("TMP");
+        if (temp != NULL) {
+          strcpyX (tmpPath, temp, 0);
+          strcatX (tmpPath, "\\", 0);
+        }
+      }
+#  else 
+#    if defined(vms)
+       strcpyX (tmpPath, "SYS$SCRATCH:", 0);
+#    else 
+       strcpyX (tmpPath, vFp->path, 0);
+#    endif
+#  endif
+#endif       
+    }
+    strcpyX (originalPath, tmpPath, 0);
+    for (i = 1; i <= MAX_TMP; i++) {
+       snprintf (EofS(tmpPath), (size_t) pathLength + 1 - strlen(tmpPath),
+                 "TMP%05ld.%s", (long)(rand()%100000), vFp->scratchExt);
+       if (!IsReg(tmpPath)) {
+	 fp = OpenFile (tmpPath, WRITE_PLUS_a_mode);
+	 if (fp != NULL) break; 
+       }
+       strcpyX (tmpPath, originalPath, 0);
+    }
+    cdf_FreeMemory (originalPath, NULL);
+    if (fp == NULL) {
       cdf_FreeMemory (tmpPath, NULL);
       return FALSE;
+    } else {
+      cdf_FreeMemory (vFp->path, NULL);
+      vFp->path = tmpPath;
+      vFp->fp = fp;
     }
+    cdf_FreeMemory (originalPath, NULL);
   }
   /****************************************************************************
   * Tally a block write.
@@ -741,7 +786,6 @@ char *extension;	/* Extension to use for the scratch file.  If NULL,
   vFp->GDR64 = NULL;
   vFp->ADRList = NULL;
   vFp->ADRList64 = NULL;
-
   /****************************************************************************
   * Return pointer to vFILE structure.
   ****************************************************************************/
