@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                                            Virtual stream file.
 *
@@ -77,6 +82,8 @@ BOO(useVmem,LoadVMemory((MemHandle)cache->ptr,TRUE),cache->ptr)
 #define CACHEbufferWRITEto(cache) cache->ptr
 #endif
 
+int pid = -1;
+
 /******************************************************************************
 * Local function prototypes.
 ******************************************************************************/
@@ -85,8 +92,10 @@ static int OpenFile PROTOARGs((char *file_spec, char *a_mode));
 #else
 static FILE *OpenFile PROTOARGs((char *file_spec, char *a_mode));
 #endif
+
 static Logical FreeCache PROTOARGs((vCACHE *firstCache));
 static vCACHE *FindCache PROTOARGs((vFILE *vFp, long blockN));
+static int CountCaches PROTOARGs((vFILE *vFp));
 static Logical vRead PROTOARGs((
   OFF_T offset, void *buffer, size_t nBytes, vFILE *vFp
 ));
@@ -101,6 +110,7 @@ static Logical WriteBlockFromCache PROTOARGs((
 static Logical WriteBlockFromBuffer PROTOARGs((
   vFILE *vFp, long blockN, void *buffer, size_t Nbytes
 ));
+
 #if EXTEND_FILE
 static Logical ExtendFile PROTOARGs((vFILE *vFp, long toBlockN));
 #endif
@@ -140,7 +150,7 @@ char *a_mode;
 }   
 
 /******************************************************************************
-* FindCache.
+* FindCache. If it is found, then move it to the head.
 ******************************************************************************/
 
 static vCACHE *FindCache (vFp, blockN)
@@ -171,6 +181,22 @@ long blockN;
   return NULL;
 }
 
+/******************************************************************************
+* CountCaches.
+******************************************************************************/
+
+static int CountCaches (vFp)
+vFILE *vFp;
+{
+  int ix = 0;
+  vCACHE *cache = vFp->cacheHead;
+  while (cache != NULL) {
+    ++ix;
+    cache = cache->next;
+  }
+  return ix;
+}
+
 /******************************************************************************
 * FlushCache64.
 * Write cache buffers to disk from the specified starting buffer to the last
@@ -244,14 +270,14 @@ vFILE *vFp;
   * reached.
   ****************************************************************************/
   if (vFp->nBuffers < vFp->maxBuffers) {
-    cache = (vCACHE *) cdf_AllocateMemory (sizeof(vCACHE), NULL);
+    cache = (vCACHE *) cdf_AllocateMemory ((size_t)sizeof(vCACHE), NULL);
     if (cache != NULL) {
 #if defined(MICROSOFTC_700) && INCLUDEvMEMORY
       if (useVmem)
 	cache->ptr = (void *) AllocateVMemory (nCACHE_BUFFER_BYTEs);
       else
 #endif
-	cache->ptr = cdf_AllocateMemory (nCACHE_BUFFER_BYTEs, NULL);
+	cache->ptr = cdf_AllocateMemory ((size_t)nCACHE_BUFFER_BYTEs, NULL);
       if (cache->ptr != NULL) {
 	if (vFp->cacheHead == NULL) {
 	  vFp->cacheHead = cache;
@@ -359,7 +385,7 @@ long toBlockN;
 	cache->modified = FALSE;
       }
       else {
-	Byte buffer[nCACHE_BUFFER_BYTEs];
+	Byte1 buffer[nCACHE_BUFFER_BYTEs];
 	if (!vRead(((OFF_T)lastPhyBlockN) * nCACHE_BUFFER_BYTEs,
 		   buffer,(size_t)nBytes,vFp)) return FALSE;
 #if CLEAR_BYTES
@@ -384,7 +410,7 @@ long toBlockN;
        cache->modified = FALSE;
      }
      else {
-       Byte buffer[nCACHE_BUFFER_BYTEs];
+       Byte1 buffer[nCACHE_BUFFER_BYTEs];
 #if CLEAR_BYTES
        ClearBytes (buffer, 0, nCACHE_BUFFER_BYTEs - 1);
 #endif
@@ -532,6 +558,20 @@ vFILE *vFp;
 #endif
 #if defined(win32) && (_FILE_OFFSET_BITS == 64)
   int fh;
+#else
+  FILE *fp;
+#endif
+#if defined(win32)
+  char *temp;
+#endif
+#if defined(unix)
+  int    ret, endian = 1;
+  long long tmpx;
+  union {
+    long long tmpTime;
+    unsigned int tmp2[2];
+  } totmp;
+  struct timeval nowtime;
 #endif
   /****************************************************************************
   * Create the scratch file if necessary.  If so, the current file path is
@@ -542,43 +582,91 @@ vFILE *vFp;
 #else
   if (vFp->fp == NULL) {
 #endif
-    long i; char *tmpPath; size_t pathLength;
+    long i; char *tmpPath, *originalPath; size_t pathLength;
     pathLength = strlen(vFp->path) + 1 + 8 + 1 + EXT_LEN;
-    tmpPath = (char *) cdf_AllocateMemory (pathLength + 1, NULL);
+    if (pathLength < DU_MAX_DIR_LEN) pathLength = DU_MAX_DIR_LEN;
+    tmpPath = (char *) cdf_AllocateMemory ((size_t)pathLength + 1, NULL);
     if (tmpPath == NULL) return FALSE;
+    originalPath = (char *) cdf_AllocateMemory ((size_t)pathLength + 1, NULL);
+    if (originalPath == NULL) return FALSE;
+    if (pid == -1) {
+#if defined(win32)
+      pid = (int) _getpid();
+#else
+      pid = (int) getpid();
+#endif
+#if defined(unix)
+      ret = gettimeofday(&nowtime, NULL);
+      tmpx = (long long) nowtime.tv_sec*1000000LL + (int) nowtime.tv_usec;
+      tmpx = tmpx % 10000000000000LL;
+      totmp.tmpTime = (long long) pid * tmpx;
+      srand((unsigned int)(*(char *)&endian==1)?totmp.tmp2[0]:totmp.tmp2[1]);
+#else
+      srand((unsigned int)pid*time(NULL));
+#endif
+    }
+    if (vFp->path != NULL && strlen(vFp->path) != 0) { 
+      strcpyX (tmpPath, vFp->path, 0);
+      AppendToDir (tmpPath, "");
+    } else {
+      /* Use the known tmp direcotry */
+#if defined(linux) || defined(unix) || defined(__CYGWIN__) || \
+    defined(__MINGw32__)
+      strcpyX (tmpPath, "/tmp/", 0);
+#else
+#  if defined(win32)
+      temp = getenv("TEMP");
+      if (temp != NULL) {
+        strcpyX (tmpPath, temp, 0);
+        strcatX (tmpPath, "\\", 0);
+      } else {
+        temp = getenv("TMP");
+        if (temp != NULL) {
+          strcpyX (tmpPath, temp, 0);
+          strcatX (tmpPath, "\\", 0);
+        }
+      }
+#  else
+#    if defined(vms)
+      strcpyX (tmpPath, "SYS$SCRATCH:", 0);
+#    else
+      strcpyX (tmpPath, vFp->path, 0);
+#    endif
+#  endif
+#endif
+    }
+    strcpyX (originalPath, tmpPath, 0);
     for (i = 1; i <= MAX_TMP; i++) {
-       strcpyX (tmpPath, vFp->path, 0);
-       AppendToDir (tmpPath, "");
        snprintf (EofS(tmpPath), (size_t) pathLength + 1 - strlen(tmpPath),
-                 "TMP%05ld.%s", i, vFp->scratchExt);
+                 "TMP%05ld.%s", (long)(rand()%100000), vFp->scratchExt);
        if (!IsReg(tmpPath)) {
 #if defined(win32) && (_FILE_OFFSET_BITS == 64)
          fh = OpenFile (tmpPath, WRITE_PLUS_a_mode);
-         if (fh == -1) {
+         if (fh != -1) break; 
 #else
-	 FILE *fp = OpenFile (tmpPath, WRITE_PLUS_a_mode);
-	 if (fp == NULL) {
+	 fp = OpenFile (tmpPath, WRITE_PLUS_a_mode);
+	 if (fp != NULL) break;
 #endif
-	   cdf_FreeMemory (tmpPath, NULL);
-	   return FALSE;
-	 }
-	 cdf_FreeMemory (vFp->path, NULL);
-	 vFp->path = tmpPath;
-#if defined(win32) && (_FILE_OFFSET_BITS == 64)
-	 vFp->fh = fh;
-#else
-	 vFp->fp = fp;
-#endif
-	 break;
        }
+       strcpyX (tmpPath, originalPath, 0);
+/*       srand((unsigned int)pid*(time(NULL)+i-1)); */
     }
+    cdf_FreeMemory (originalPath, NULL);
 #if defined(win32) && (_FILE_OFFSET_BITS == 64)
-    if (vFp->fh == -1) {
+    if (fh == -1) {
 #else
-    if (vFp->fp == NULL) {	   /* Hardly seems likely but we'll check... */
+    if (fp == NULL) {
 #endif
       cdf_FreeMemory (tmpPath, NULL);
       return FALSE;
+    } else {
+      cdf_FreeMemory (vFp->path, NULL);
+      vFp->path = tmpPath;
+#if defined(win32) && (_FILE_OFFSET_BITS == 64)
+      vFp->fh = fh;
+#else
+      vFp->fp = fp;
+#endif
     }
   }
   /****************************************************************************
@@ -660,12 +748,12 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
 VISIBLE_PREFIX size_t V_read64 (buffer, item_size, n_items, vFp)
 void *buffer;           /* Pointer to buffer. */
 size_t item_size;       /* Size (in bytes) of each item to read. */
-size_t n_items;          /* Number of items to read. */
+size_t n_items;         /* Number of items to read. */
 vFILE *vFp;             /* Pointer to vFILE structure. */
 {
   size_t nBytesX;       /* Total number of bytes in buffer. */
   size_t nBytes;	/* Total number of bytes to read. */
-  OFF_T  remainingItems;	/* Number of items remaining after the offset. */
+  OFF_T  remainingItems;/* Number of items remaining after the offset. */
   size_t nItems;	/* Number of items to actually be read. */
   long firstBlockN;    	/* First block involved in read. */
   long lastBlockN;     	/* Last block involved in read. */
@@ -675,7 +763,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
   long blockN;         	/* Block number in file (from 0). */
   long atBlockN;       	/* Block number in file (from 0) at which to read. */
   vCACHE *cache;        /* Pointer to cache structure. */
-  Byte *cBuffer;        /* Pointer to cache buffer. */
+  Byte1 *cBuffer;       /* Pointer to cache buffer. */
   int remainingBytes;	/* Number of bytes remaining in a block. */
   /****************************************************************************
   * Validate read.
@@ -724,7 +812,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
       return 0;
     }
     memmove (buffer, cBuffer + bufferOffset, (size_t) xBytes);
-    buffer = (Byte *) buffer + xBytes;
+    buffer = (Byte1 *) buffer + xBytes;
     atBlockN = firstBlockN + 1;
   }
   else
@@ -734,11 +822,6 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
   ****************************************************************************/
   lastBlockN = (long) ((vFp->offset64 + (OFF_T) nBytes - 1) / 
 			(OFF_T) nCACHE_BUFFER_BYTEs); 
-/*
-  tmpBlockN = ((OFF_T) (vFp->offset64 + nBytes - 1)) / nCACHE_BUFFER_BYTEs;
-  if (tmpBlockN > maxCount) lastBlockN = 0;
-  else lastBlockN = (long) tmpBlockN;
-*/
   for (blockN = atBlockN; blockN <= lastBlockN; blockN++) {
      xBytes = (size_t) (((OFF_T)vFp->offset64) + (OFF_T) nBytes - 
 		        ((OFF_T) blockN) * nCACHE_BUFFER_BYTEs);
@@ -774,8 +857,9 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
 	   return 0;
 	 }
        }
+
      }
-     buffer = (Byte *) buffer + xBytes;
+     buffer = (Byte1 *) buffer + xBytes;
   }
   /****************************************************************************
   * Increment current file offset or set to EOF if the EOF indicator was set.
@@ -803,7 +887,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
   long atBlockN;        /* Block number in file (from 0) at which to write. */
   size_t xBytes;        /* Number of bytes in a transfer. */
   vCACHE *cache;        /* Pointer to cache structure. */
-  Byte *cBuffer;        /* Pointer to cache buffer. */
+  Byte1 *cBuffer;       /* Pointer to cache buffer. */
   size_t nBytesInBlock;	/* Number of bytes to the end of the block. */
   /****************************************************************************
   * Validate write.
@@ -826,11 +910,6 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
   * the cache (even if a full block).
   ****************************************************************************/
   firstBlockN = (long) (vFp->offset64 / (OFF_T) nCACHE_BUFFER_BYTEs); 
-/*
-  tmpBlockN = (OFF_T) (vFp->offset64 / nCACHE_BUFFER_BYTEs);
-  if (tmpBlockN > maxCount) firstBlockN = 0;
-  else firstBlockN = (long) tmpBlockN; 
-*/
   bufferOffset = (int) ((OFF_T) vFp->offset64 % nCACHE_BUFFER_BYTEs);
   nBytesInBlock = nCACHE_BUFFER_BYTEs - bufferOffset;
   xBytes = MINIMUM (nBytes, nBytesInBlock);
@@ -873,7 +952,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
     cache->modified = TRUE;
     vFp->length64 = MaxLongLong (vFp->length64, vFp->offset64 + 
 				  (OFF_T) xBytes);
-    buffer = (Byte *) buffer + xBytes;
+    buffer = (Byte1 *) buffer + xBytes;
     atBlockN = firstBlockN + 1;
   }
   else
@@ -883,11 +962,6 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
   ****************************************************************************/
   lastBlockN = (long) ((vFp->offset64 + (OFF_T) nBytes - 1) / 
 			(OFF_T) nCACHE_BUFFER_BYTEs); 
-/*  tmpBlockN = ((OFF_T) (((OFF_T)vFp->offset64) + nBytes - 1)) / 
-			nCACHE_BUFFER_BYTEs;
-  if (tmpBlockN > maxCount) lastBlockN = 0;
-  else lastBlockN = (long) tmpBlockN;
-*/
   for (blockN = atBlockN; blockN <= lastBlockN; blockN++) {
      xBytes = (size_t) (((OFF_T)vFp->offset64) + (OFF_T) nBytes - 
 		        ((OFF_T) blockN) * nCACHE_BUFFER_BYTEs);
@@ -957,7 +1031,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
      }
      vFp->length64 = MaxLongLong (vFp->length64, ((OFF_T) blockN) *
 			          nCACHE_BUFFER_BYTEs + (OFF_T) xBytes);
-     buffer = (Byte *) buffer + xBytes;
+     buffer = (Byte1 *) buffer + xBytes;
   }
   /****************************************************************************
   * Increment current file offset.
@@ -1048,7 +1122,7 @@ char *a_mode;           /* Access mode. */
   /****************************************************************************
   * Allocate and load vFILE structure.
   ****************************************************************************/
-  vFp = (vFILE *) cdf_AllocateMemory (sizeof(vFILE), NULL);
+  vFp = (vFILE *) cdf_AllocateMemory ((size_t)sizeof(vFILE), NULL);
   if (vFp == NULL) {
     FCLOSE64 (fp);
     return NULL;
@@ -1059,7 +1133,7 @@ char *a_mode;           /* Access mode. */
 #else
   vFp->fp = fp;
 #endif
-  vFp->path = (char *) cdf_AllocateMemory (strlen(file_spec) + 1, NULL);
+  vFp->path = (char *) cdf_AllocateMemory ((size_t)strlen(file_spec) + 1, NULL);
   if (vFp->path == NULL) {
     cdf_FreeMemory (vFp, NULL);
     FCLOSE64 (fp);
@@ -1395,3 +1469,24 @@ vSTATS *vStats;         /* Pointer to statistics structure. */
   return BOO(error,EOF,0);
 }
 
+
+/******************************************************************************
+* GetMyPID.
+* Returns the PID of the session. If -1 then it is not set.
+******************************************************************************/
+
+VISIBLE_PREFIX int GetMyPID ()
+{
+  return pid;
+}
+
+/******************************************************************************
+* SetMyPID.
+* Sets the PID of the session.
+******************************************************************************/
+
+VISIBLE_PREFIX void SetMyPID (int npid)
+{
+  pid = npid;
+}
+
