@@ -296,7 +296,7 @@ switch (argC) {
     break;
 }
 
-CDFsetValidate (VALIDATEFILEon);  
+/* CDFsetValidate (VALIDATEFILEon); */
 /******************************************************************************
 * Create skeleton table.
 ******************************************************************************/
