@@ -55,7 +55,7 @@ MAIN {
 ******************************************************************************/
 
 Logical DumpCDF (argC, argV)
-int argC;
+int  argC;
 char *argV[];
 {
 
@@ -291,14 +291,14 @@ switch (argC) {
     fprintf (fp, "\"...\n");
   }
 
-  CDFsetValidate (VALIDATEFILEon);
+  if (detect) CDFsetValidate (VALIDATEFILEon);
   /****************************************************************************
   * Open/inquire CDF.
   ****************************************************************************/
 
   status = CDFlib (OPEN_, CDF_, CDFpath, &id,
                    NULL_);
-  if (status < CDF_OK) QuitCDF (CDFpath, "1.0", status, CDFpath);
+  if (status < CDF_OK) return QuitCDF (CDFpath, "1.0", status, CDFpath);
 
   status = CDFlib (SELECT_, CDF_, id,
                             CDF_READONLY_MODE_, READONLYon,
@@ -317,7 +317,7 @@ switch (argC) {
                          CDF_COPYRIGHT_, copyRight,
                          CDF_CHECKSUM_, &checksum,
                    NULL_);
-  if (status < CDF_OK) QuitCDF (CDFpath, "2.0", status, CDFpath);
+  if (status < CDF_OK) return QuitCDF (CDFpath, "2.0", status, CDFpath);
 
   /**********************************/
   /* Print out the file information */
@@ -345,7 +345,7 @@ switch (argC) {
       if (!PriorTo ("3.5.0", version, release, increment)) {
         status = CDFlib (GET_, CDF_LEAPSECONDLASTUPDATED_, &lastUpdated,
                          NULL_);
-        if (status < CDF_OK) QuitCDF (CDFpath, "2.1", status, CDFpath);
+        if (status < CDF_OK) return QuitCDF (CDFpath, "2.1", status, CDFpath);
         fprintf (fp, "LeapSecondLastUpdated:     %ld\n", lastUpdated);
       }
   }
@@ -369,14 +369,14 @@ switch (argC) {
         status = CDFlib (SELECT_, ATTR_, (long) i,
                          GET_, ATTR_SCOPE_, &scope,
                          NULL_);
-        if (status < CDF_OK) QuitCDF (CDFpath, "3.0", status, NULL);
+        if (status < CDF_OK) return QuitCDF (CDFpath, "3.0", status, NULL);
 
         if (scope != GLOBAL_SCOPE) continue;
 
         status = CDFlib (GET_, ATTR_NAME_, attrName,
                                ATTR_NUMgENTRIES_, &numgEntries,
                          NULL_);
-        if (status < CDF_OK) QuitCDF (CDFpath, "4.0", status, attrName);
+        if (status < CDF_OK) return QuitCDF (CDFpath, "4.0", status, attrName);
 
         if (numgEntries <= 1) {
           if (!detect) fprintf (fp, "%s (%ld entry):\n", attrName, numgEntries);
@@ -400,7 +400,7 @@ switch (argC) {
               value = cdf_AllocateMemory ((size_t) nBytes, FatalError);
               status = CDFlib (GET_, gENTRY_DATA_, value,
                                NULL_);
-              if (status < CDF_OK) QuitCDF (CDFpath, "5.0", status, attrName);
+              if (status < CDF_OK) return QuitCDF (CDFpath, "5.0", status, attrName);
 	      if (detect) {
 		++k; ++j;
                 cdf_FreeMemory (value, FatalError);
@@ -455,7 +455,7 @@ switch (argC) {
         status = CDFlib (SELECT_, ATTR_, (long) i,
                          GET_, ATTR_SCOPE_, &scope,
                          NULL_);
-        if (status < CDF_OK) QuitCDF (CDFpath, "6.0", status, NULL);
+        if (status < CDF_OK) return QuitCDF (CDFpath, "6.0", status, NULL);
 
         if (scope == GLOBAL_SCOPE) continue;
 
@@ -463,7 +463,7 @@ switch (argC) {
                                ATTR_NUMrENTRIES_, &numrEntries,
                                ATTR_NUMzENTRIES_, &numzEntries,
                          NULL_);
-        if (status < CDF_OK) QuitCDF (CDFpath, "7.0", status, attrName);
+        if (status < CDF_OK) return QuitCDF (CDFpath, "7.0", status, attrName);
 
         if (!detect) fprintf (fp, "%s\n", attrName);
       }
@@ -490,7 +490,7 @@ switch (argC) {
     status = CDFlib (GET_, rVARs_NUMDIMS_, &numDims,
                            rVARs_DIMSIZES_, dimSizes,
                      NULL_);
-    if (status < CDF_OK) QuitCDF (CDFpath, "8.0", status, NULL);
+    if (status < CDF_OK) return QuitCDF (CDFpath, "8.0", status, NULL);
 
     for (i = 0; i < numrVars; i++) {
        status = CDFlib (SELECT_, rVAR_, (long) i,
@@ -501,7 +501,7 @@ switch (argC) {
                               rVAR_RECVARY_, &recVary,
                               rVAR_SPARSERECORDS_, &sp,
                         NULL_);
-      if (status < CDF_OK) QuitCDF (CDFpath, "9.0", status, varName);
+      if (status < CDF_OK) return QuitCDF (CDFpath, "9.0", status, varName);
       if (detect) continue;
       if (numVars > 0) {
         ifound = FALSE;
@@ -564,7 +564,7 @@ switch (argC) {
                               zVAR_RECVARY_, &recVary,
                               zVAR_SPARSERECORDS_, &sp,
                         NULL_);
-      if (status < CDF_OK) QuitCDF (CDFpath, "10.0", status, varName);
+      if (status < CDF_OK) return QuitCDF (CDFpath, "10.0", status, varName);
       if (detect) continue;
       if (numVars > 0) {
         ifound = FALSE;
@@ -635,10 +635,11 @@ switch (argC) {
     if (ii >= (int) numrVars) ii = ii - (int) numrVars;
     status = CDFlib (SELECT_, BOO(Z,zVAR_,rVAR_), (long) ii,
                      GET_, BOO(Z,zVAR_NAME_,rVAR_NAME_), varName,
-                           BOO(Z,zVAR_COMPRESSION_,rVAR_COMPRESSION_), 
+                           BOO(Z,zVAR_COMPRESSION_,rVAR_COMPRESSION_),
                                &compression, compressParms, &cPct,
+			   BOO(Z,zVAR_MAXREC_,rVAR_MAXREC_), &maxRec,
                      NULL_);
-    if (status < CDF_OK) QuitCDF (CDFpath, "11.0", status, varName);
+    if (status < CDF_OK) return QuitCDF (CDFpath, "11.0", status, varName);
         
     if (numVars > 0) {
       ifound = FALSE;
@@ -652,11 +653,11 @@ switch (argC) {
     }
 
     if (!detect) {
-      fprintf (fp, "%s (%d)", varName, ii);
+      fprintf (fp, "%s (No: %d) (Recs: %ld)", varName, ii, (maxRec+1));
       if (METADATADUMP && compression != NO_COMPRESSION) {
-        fprintf (fp, " (%s)\n",
+        fprintf (fp, " (Compression: %s)\n",
                  CompressionToken(compression, compressParms));
-        Ncharacters (fp, (int) strlen(varName)+9, '-');
+        Ncharacters (fp, (int) strlen(varName), '-');
        } else {
         fprintf (fp, "\n");
         Ncharacters (fp, (int) strlen(varName), '-');
@@ -676,7 +677,6 @@ switch (argC) {
                            BOO(Z,zVAR_DIMVARYS_,rVAR_DIMVARYS_), dimVarys,
                            BOO(Z,zVAR_RECVARY_,rVAR_RECVARY_), &recVary,
                            BOO(Z,zVAR_NUMRECS_,rVAR_NUMRECS_), &numRecs,
-                           BOO(Z,zVAR_MAXREC_,rVAR_MAXREC_), &maxRec,
                            BOO(Z,zVAR_MAXallocREC_,rVAR_MAXallocREC_), 
                                &maxAllocRec,
                            BOO(Z,zVAR_NUMallocRECS_,rVAR_NUMallocRECS_), 
@@ -686,7 +686,7 @@ switch (argC) {
                            BOO(Z,zVAR_SPARSERECORDS_,rVAR_SPARSERECORDS_), 
                                &sp,
                      NULL_);
-    if (status < CDF_OK) QuitCDF (CDFpath, "12.0", status, varName);
+    if (status < CDF_OK) return QuitCDF (CDFpath, "12.0", status, varName);
 
     if ((dump != METADATADUMP && dump != VARIABLEDUMP) || detect) {
       if (STRINGdataType(dataType)) imore = 1;
@@ -781,7 +781,7 @@ switch (argC) {
           status = CDFlib (SELECT_, ATTR_, (long) ij,
                            GET_, ATTR_SCOPE_, &scope,
                            NULL_);
-          if (status < CDF_OK) QuitCDF (CDFpath, "14.0", status, NULL);
+          if (status < CDF_OK) return QuitCDF (CDFpath, "14.0", status, NULL);
 
           if (scope == GLOBAL_SCOPE) continue;
 
@@ -800,7 +800,7 @@ switch (argC) {
           status = CDFlib (GET_, ATTR_NAME_, attrName,
                                  BOO(Z,zENTRY_DATA_,rENTRY_DATA_), value,
                            NULL_);
-          if (status < CDF_OK) QuitCDF (CDFpath, "15.0", status, attrName);
+          if (status < CDF_OK) return QuitCDF (CDFpath, "15.0", status, attrName);
  
           fprintf (fp, "     %s", attrName);
           ir = (int) strlen(attrName);
@@ -895,7 +895,7 @@ switch (argC) {
           for (j = 0; j < numDims; ++j)
             if (!dimVarys[j]) hyperX.dimCounts[j] = 1;
           status = HYPER_READ (id, Z, hyperX, buffer);
-          if (status < CDF_OK) QuitCDF(CDFpath, "16.0", status, varName);
+          if (status < CDF_OK) return QuitCDF(CDFpath, "16.0", status, varName);
           if (!detect) {
             offset = 0;
             for (k = 0, value = buffer; k < nValues;
@@ -995,7 +995,7 @@ switch (argC) {
                                  BOO(Z,zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_), 
                                  dimCounts,
                         NULL_);
-        if (status < CDF_OK) QuitCDF(CDFpath, "17.0", status, varName);
+        if (status < CDF_OK) return QuitCDF(CDFpath, "17.0", status, varName);
         if (toCol2Row) value2 = malloc((size_t)nLogValuesPerRec*nValueBytes[0]);
         for (k = 0; k < (int) (maxRec+1); k++) {
           status = CDFlib(SELECT_, BOO(Z,zVAR_RECNUMBER_,rVARs_RECNUMBER_), 
@@ -1003,7 +1003,7 @@ switch (argC) {
                           GET_, BOO(Z,zVAR_HYPERDATA_,rVAR_HYPERDATA_), buffer,
                           NULL_);
 /*          if (status == VIRTUAL_RECORD_DATA) continue; */
-          if (status < CDF_OK) QuitCDF(CDFpath, "18.0", status, varName);
+          if (status < CDF_OK) return QuitCDF(CDFpath, "18.0", status, varName);
 	  if (!detect) {
             offset = 0;
             for (ir = 0, value = buffer; ir < nLogValuesPerRec;
@@ -1107,7 +1107,7 @@ switch (argC) {
 * QuitCDF.
 ******************************************************************************/
 
-void QuitCDF (file, where, status, msg)
+Logical QuitCDF (file, where, status, msg)
 char *file;
 char *where;
 CDFstatus status;
@@ -1119,13 +1119,13 @@ char *msg;
     CDFlib (SELECT_, CDF_STATUS_, status,
             GET_, STATUS_TEXT_, text,
             NULL_);
-    printf ("%s ", text);
+    printf ("ERROR> %s ", text);
     if (msg != NULL) printf ("(%s)\n", msg);
     else printf("\n");
   }
   CDFlib (CLOSE_, CDF_,
           NULL_);
   printf ("\n");
-  ExitBAD;
+  return FALSE;
 }
 
