@@ -250,7 +250,7 @@ char *argV[];
    /***************************************************************************
    * Edit CDF(s).
    ***************************************************************************/
-   CDFsetValidate (VALIDATEFILEon); 
+/*   CDFsetValidate (VALIDATEFILEon); */
    if (promptForSpec)
      status = SpecificationPrompt (CDFspec, useFormat, negToPosFp0, zMode,
 				   workingCache, stageCache, compressCache);
@@ -1300,7 +1300,7 @@ struct ItemWindowStruct *IW;
   snprintf (EofS(IW->hLines[2]),
 	    (size_t) sizeof(BLANKs78)-strlen(IW->hLines[2]),
 	    "Checksum:   %s", ChecksumToken(checksum));
-  if (TT2000updated >= 0) {
+  if (!PriorTo ("3.6.0", version, release, increment)) {
     pad = MAIN_MENU_2nd_THIRD_LEN - strlen(IW->hLines[2]);
     if (pad > 0) CatNcharacters (IW->hLines[2], pad, (int) ' ');
     snprintf (EofS(IW->hLines[2]),
