@@ -1,5 +1,5 @@
 /******************************************************************************
-* Copyright 1996-2013 United States Government as represented by the
+* Copyright 1996-2014 United States Government as represented by the
 * Administrator of the National Aeronautics and Space Administration.
 * All Rights Reserved.
 ******************************************************************************/
@@ -108,7 +108,7 @@ char *argV[];
   };
   static char *reportTokens[] = { "errors", "warnings", "informationals" };
 
-  CDFsetValidate (VALIDATEFILEon);
+/*  CDFsetValidate (VALIDATEFILEon); */
   /****************************************************************************
   * Get qualifiers/options/parameters.
   ****************************************************************************/
@@ -119,7 +119,7 @@ char *argV[];
     case 2:
       if (strcmp(argV[1],"-java") == 0) {
         pagingOn = FALSE;
-        PageOLH ("cdfstatj.olh", argV[0]);
+        PageOLH ("cdfstatsj.olh", argV[0]);
         return TRUE;
       }
     default:
