@@ -1,5 +1,5 @@
 /******************************************************************************
-* Copyright 1996-2013 United States Government as represented by the
+* Copyright 1996-2014 United States Government as represented by the
 * Administrator of the National Aeronautics and Space Administration.
 * All Rights Reserved.
 ******************************************************************************/
@@ -206,6 +206,10 @@ char *argV[];
     free (CDFspec[i]);
   }
   free (CDFspec);
-  return TRUE; 
+  if (numParms == 1) {
+    if (status < CDF_OK) return FALSE;
+    else return TRUE;
+  } else
+    return TRUE; 
 }
 
